
from __future__ import absolute_import

from json import dumps

from core import output
from core.config import CONFIG
from core.tools import to_bytes

from hpfeeds.twisted import ClientSessionService

from twisted.internet import endpoints, reactor, ssl

class Output(output.Output):

    def start(self):
        self.channel = CONFIG.get('output_hpfeed', 'channel', fallback='siphoney')

        if CONFIG.has_option('output_hpfeed', 'endpoint'):
            endpoint = CONFIG.get('output_hpfeed', 'endpoint')
        else:
            server = CONFIG.get('output_hpfeed', 'server')
            port = CONFIG.getint('output_hpfeed', 'port')

            if CONFIG.has_option('output_hpfeed', 'tlscert'):
                with open(CONFIG.get('output_hpfeed', 'tlscert')) as fp:
                    authority = ssl.Certificate.loadPEM(fp.read())
                options = ssl.optionsForClientTLS(server, authority)
                endpoint = endpoints.SSL4ClientEndpoint(reactor, server, port, options)
            else:
                endpoint = endpoints.HostnameEndpoint(reactor, server, port)

        ident = CONFIG.get('output_hpfeed', 'identifier', raw=True)
        secret = CONFIG.get('output_hpfeed', 'secret', raw=True)

        self.client = ClientSessionService(endpoint, ident, secret)
        self.client.startService()

    def stop(self):
        self.client.stopService()

    def write(self, event):
        self.client.publish(self.channel, to_bytes(dumps(event, sort_keys=True)))




from __future__ import absolute_import

from core import output
from core.config import CONFIG
from core.tools import geolocate

from geoip2.database import Reader
from sqlite3 import OperationalError, IntegrityError

from twisted.enterprise.adbapi import ConnectionPool
from twisted.python.log import msg


class Output(output.Output):

    def start(self):
        db_name = CONFIG.get('output_sqlite', 'db_file', fallback='data/siphoney.db')
        self.debug = CONFIG.getboolean('output_sqlite', 'debug', fallback=False)
        self.geoip = CONFIG.getboolean('output_sqlite', 'geoip', fallback=True)

        try:
            self.db = ConnectionPool(
                'sqlite3',
                database=db_name,
                check_same_thread=False
            )
        except OperationalError as e:
            msg(e)

        self.db.start()

        if self.geoip:
            geoipdb_city_path = CONFIG.get('output_sqlite', 'geoip_citydb', fallback='data/GeoLite2-City.mmdb')
            geoipdb_asn_path = CONFIG.get('output_sqlite', 'geoip_asndb', fallback='data/GeoLite2-ASN.mmdb')

            try:
                self.reader_city = Reader(geoipdb_city_path)
            except Exception:
                self.reader_city = None
                msg('Failed to open City GeoIP database {}'.format(geoipdb_city_path))
            try:
                self.reader_asn = Reader(geoipdb_asn_path)
            except Exception:
                self.reader_asn = None
                msg('Failed to open ASN GeoIP database {}'.format(geoipdb_asn_path))

    def stop(self):
        if self.geoip:
            if self.reader_city is not None:
                self.reader_city.close()
            if self.reader_asn is not None:
                self.reader_asn.close()

    def write(self, event):
        self.db.runInteraction(self.connect_event, event)

    def simple_query(self, txn, sql, args):
        if self.debug:
            if len(args):
                msg('output_sqlite: SQLite3 query: {} {}'.format(sql, repr(args)))
            else:
                msg('output_sqlite: SQLite3 query: {}'.format(sql))
        try:
            if len(args):
                txn.execute(sql, args)
            else:
                txn.execute(sql)
            result = txn.fetchall()
        except IntegrityError as e:
            result = None
        except Exception as e:
            msg('output_sqlite: SQLite3 Error: {}'.format(e))
            result = None
        return result

    def get_id(self, txn, table, column, entry):
        r = self.simple_query(txn, "SELECT `id` FROM `{}` WHERE `{}` = ?".format(table, column), (entry, ))
        if r:
            id = r[0][0]
        else:
            self.simple_query(txn, "INSERT INTO `{}` (`{}`) VALUES (?)".format(table, column), (entry, ))
            r = self.simple_query(txn, 'SELECT LAST_INSERT_ROWID()', ())
            if r:
                id = int(r[0][0])
            else:
                id = 0
        return id

    def get_request_id(self, txn, headers):
        accept_id = allow_id = callids_id = contact_id = conlen_id = contype_id = cseq_id = expires_id = from_id = forward_id = supported_id = to_id = agent_id = via_id = auth_id = 0
        for key, value in headers.items():
            if key == 'accept':
                accept_id = self.get_id(txn, 'accepts', 'value', value)
            elif key == 'allow':
                allow_id = self.get_id(txn, 'allows', 'value', value)
            elif key == 'call-id':
                callids_id = self.get_id(txn, 'callids', 'value', value)
            elif key == 'contact':
                contact_id = self.get_id(txn, 'contacts', 'value', value)
            elif key == 'content-length':
                conlen_id = self.get_id(txn, 'con_lens', 'value', value)
            elif key == 'content-type':
                contype_id = self.get_id(txn, 'con_types', 'value', value)
            elif key == 'cseq':
                cseq_id = self.get_id(txn, 'cseqs', 'value', value)
            elif key == 'expires':
                expires_id = self.get_id(txn, 'expires', 'value', value)
            elif key == 'from':
                from_id = self.get_id(txn, 'froms', 'value', value)
            elif key == 'max-forwards':
                forward_id = self.get_id(txn, 'forwards', 'value', value)
            elif key == 'supported':
                supported_id = self.get_id(txn, 'supporteds', 'value', value)
            elif key == 'to':
                to_id = self.get_id(txn, 'tos', 'value', value)
            elif key == 'user-agent':
                agent_id = self.get_id(txn, 'agents', 'value', value)
            elif key == 'via':
                via_id = self.get_id(txn, 'vias', 'value', value)
            elif key == 'authorization':
                auth_id = self.get_id(txn, 'authorizations', 'value', value)
            else:
                self.get_id(txn, 'unknown', 'header', key)
        self.simple_query(txn,
            """
            INSERT INTO `requests` (`accept`, `allow`, `callid`, `contact`, `con_len`, `con_type`, `cseq`, `expires`, `from`, `forwards`, `supported`, `to`, `agent`, `via`, `authorization`)
            VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
            """,
            (accept_id, allow_id, callids_id, contact_id, conlen_id, contype_id, cseq_id, expires_id, from_id, forward_id, supported_id, to_id, agent_id, via_id, auth_id, )
        )
        r = self.simple_query(txn, 'SELECT LAST_INSERT_ROWID()', ())
        if r:
            id = int(r[0][0])
        else:
            id = 0
        return id

    def connect_event(self, txn, event):
        remote_ip = event['src_ip']
        sensor_id = self.get_id(txn, 'sensors', 'name', event['sensor'])
        message_id = self.get_id(txn, 'messages', 'value', event['message'])
        request_id = self.get_request_id(txn, event['headers'])

        self.simple_query(txn,
            """
            INSERT INTO `connections` (`timestamp`, `ip`, `remote_port`, `local_host`, `local_port`, `request`, `message`, `sensor`)
            VALUES (DATETIME(?, 'unixepoch', 'localtime'), ?, ?, ?, ?, ?, ?, ?)
            """,
            (event['unixtime'], remote_ip, event['src_port'], event['dst_ip'],
             event['dst_port'], request_id, message_id, sensor_id, ))

        if self.geoip:
            country, country_code, city, org, asn_num = geolocate(remote_ip, self.reader_city, self.reader_asn)
            result = self.simple_query(txn,
                """
                INSERT INTO `geolocation` (`ip`, `country_name`, `country_iso_code`, `city_name`, `org`, `org_asn`)
                VALUES (?, ?, ?, ?, ?, ?)
                """,
                (remote_ip, country, country_code, city, org, asn_num, ))
            if result is None:
                self.simple_query(txn,
                    """
                    UPDATE `geolocation` SET
                        `country_name` = ?,
                        `country_iso_code` = ?,
                        `city_name` = ?,
                        `org` = ?,
                        `org_asn` = ?
                    WHERE `ip` = ?
                    """,
                    (country, country_code, city, org, asn_num, remote_ip, ))




from socket import gethostname

from core.config import CONFIG


class Output(object):
    """
    Abstract base class intended to be inherited by output plugins.
    """

    def __init__(self, general_options):

        self.cfg = general_options

        if 'sensor' in self.cfg:
            self.sensor = self.cfg['sensor']
        else:
            self.sensor = CONFIG.get('honeypot', 'sensor_name', fallback=gethostname())

        self.start()

    def start(self):
        """
        Abstract method to initialize output plugin
        """
        pass

    def stop(self):
        """
        Abstract method to shut down output plugin
        """
        pass

    def write(self, event):
        """
        Handle a general event within the output plugin
        """
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import

from sys import exc_info, version_info

from core import output
from core.config import CONFIG
from core.tools import geolocate

from geoip2.database import Reader

if version_info[0] < 3:
    import pymysql
    pymysql.install_as_MySQLdb()

try:
    from MySQLdb import (Error, OperationalError)
except ImportError:
    try:
        from MySQLdb._exceptions import (Error, OperationalError)
    except ImportError:
        from _mysql_exceptions import (Error, OperationalError) # type: ignore

from twisted.enterprise.adbapi import ConnectionPool
from twisted.python.compat import reraise
from twisted.python.log import msg


class ReconnectingConnectionPool(ConnectionPool):
    """
    Reconnecting adbapi connection pool for MySQL.
    This class improves on the solution posted at
    http://www.gelens.org/2008/09/12/reinitializing-twisted-connectionpool/
    by checking exceptions by error code and only disconnecting the current
    connection instead of all of them.
    Also see:
    http://twistedmatrix.com/pipermail/twisted-python/2009-July/020007.html
    """

    def _runInteraction(self, interaction, *args, **kw):

        def rerise_exception(conn):
            _, excValue, excTraceback = exc_info()
            try:
                conn.rollback()
            except Exception:
                msg('Rollback failed')
            reraise(excValue, excTraceback)

        conn = self.connectionFactory(self)
        trans = self.transactionFactory(self, conn)
        try:
            result = interaction(trans, *args, **kw)
            trans.close()
            conn.commit()
            return result
        except OperationalError as e:
            if e.args[0] not in (2003, 2006, 2013):
                rerise_exception(conn)
            else:
                conn = self.connections.get(self.threadID())
                self.disconnect(conn)
                return ConnectionPool._runInteraction(self, interaction, *args, **kw)
        except Exception:
            rerise_exception(conn)


class Output(output.Output):

    def local_log(self, message):
        if self.debug:
            msg(message)

    def start(self):
        host = CONFIG.get('output_mysql', 'host', fallback='localhost')
        database = CONFIG.get('output_mysql', 'database', fallback='siphoney')
        user = CONFIG.get('output_mysql', 'username', fallback='siphoney', raw=True)
        password = CONFIG.get('output_mysql', 'password', fallback='', raw=True)
        port = CONFIG.getint('output_mysql', 'port', fallback=3306)

        self.debug = CONFIG.getboolean('output_mysql', 'debug', fallback=False)
        self.geoip = CONFIG.getboolean('output_mysql', 'geoip', fallback=True)

        try:
            self.dbh = ReconnectingConnectionPool(
                'MySQLdb',
                host=host,
                db=database,
                user=user,
                passwd=password,
                port=port,
                charset='utf8',
                use_unicode=True,
                cp_min=1,
                cp_max=1
            )
        except Error as e:
            self.local_log('output_mysql: MySQL Error {}: "{}"'.format(e.args[0], e.args[1]))

        if self.geoip:
            geoipdb_city_path = CONFIG.get('output_mysql', 'geoip_citydb', fallback='data/GeoLite2-City.mmdb')
            geoipdb_asn_path = CONFIG.get('output_mysql', 'geoip_asndb', fallback='data/GeoLite2-ASN.mmdb')
            try:
                self.reader_city = Reader(geoipdb_city_path)
            except Exception:
                self.reader_city = None
                self.local_log('Failed to open City GeoIP database {}'.format(geoipdb_city_path))

            try:
                self.reader_asn = Reader(geoipdb_asn_path)
            except Exception:
                self.reader_asn = None
                self.local_log('Failed to open ASN GeoIP database {}'.format(geoipdb_asn_path))

    def stop(self):
        if self.geoip:
            if self.reader_city is not None:
                self.reader_city.close()
            if self.reader_asn is not None:
                self.reader_asn.close()

    def write(self, event):
        """
        TODO: Check if the type (date, datetime or timestamp) of columns is appropriate for your needs and timezone
        - MySQL Documentation - The DATE, DATETIME, and TIMESTAMP Types
            (https://dev.mysql.com/doc/refman/5.7/en/datetime.html):
        "MySQL converts TIMESTAMP values from the current time zone to UTC for storage,
        and back from UTC to the current time zone for retrieval.
        (This does not occur for other types such as DATETIME.)"
        """
        self.dbh.runInteraction(self.connect_event, event)

    def simple_query(self, txn, sql, args):
        if self.debug:
            if len(args):
                self.local_log("output_mysql: MySQL query: {} {}".format(sql, repr(args)))
            else:
                self.local_log("output_mysql: MySQL query: {}".format(sql))
        try:
            if len(args):
                txn.execute(sql, args)
            else:
                txn.execute(sql)
            result = txn.fetchall()
        except Exception as e:
            self.local_log('output_mysql: MySQL Error: {}'.format(e))
            result = None
        return result

    def get_id(self, txn, table, column, entry):
        r = self.simple_query(txn, "SELECT `id` FROM `{}` WHERE `{}` = %s".format(table, column), (entry, ))
        if r:
            id = r[0][0]
        else:
            self.simple_query(txn, "INSERT INTO `{}` (`{}`) VALUES (%s)".format(table, column), (entry, ))
            r = self.simple_query(txn, 'SELECT LAST_INSERT_ID()', ())
            if r:
                id = int(r[0][0])
            else:
                id = 0
        return id

    def get_request_id(self, txn, headers):
        accept_id = allow_id = callids_id = contact_id = conlen_id = contype_id = cseq_id = expires_id = from_id = forward_id = supported_id = to_id = agent_id = via_id = auth_id = 0
        for key, value in headers.items():
            if key == 'accept':
                accept_id = self.get_id(txn, 'accepts', 'value', value)
            elif key == 'allow':
                allow_id = self.get_id(txn, 'allows', 'value', value)
            elif key == 'call-id':
                callids_id = self.get_id(txn, 'callids', 'value', value)
            elif key == 'contact':
                contact_id = self.get_id(txn, 'contacts', 'value', value)
            elif key == 'content-length':
                conlen_id = self.get_id(txn, 'con_lens', 'value', value)
            elif key == 'content-type':
                contype_id = self.get_id(txn, 'con_types', 'value', value)
            elif key == 'cseq':
                cseq_id = self.get_id(txn, 'cseqs', 'value', value)
            elif key == 'expires':
                expires_id = self.get_id(txn, 'expires', 'value', value)
            elif key == 'from':
                from_id = self.get_id(txn, 'froms', 'value', value)
            elif key == 'max-forwards':
                forward_id = self.get_id(txn, 'forwards', 'value', value)
            elif key == 'supported':
                supported_id = self.get_id(txn, 'supporteds', 'value', value)
            elif key == 'to':
                to_id = self.get_id(txn, 'tos', 'value', value)
            elif key == 'user-agent':
                agent_id = self.get_id(txn, 'agents', 'value', value)
            elif key == 'via':
                via_id = self.get_id(txn, 'vias', 'value', value)
            elif key == 'authorization':
                auth_id = self.get_id(txn, 'authorizations', 'value', value)
            else:
                self.get_id(txn, 'unknown', 'header', key)
        self.simple_query(txn,
            """
            INSERT INTO `requests` (`accept`, `allow`, `callid`, `contact`, `con_len`, `con_type`, `cseq`, `expires`, `from`, `forwards`, `supported`, `to`, `agent`, `via`, `authorization`)
            VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)
            """,
            (accept_id, allow_id, callids_id, contact_id, conlen_id, contype_id, cseq_id, expires_id, from_id, forward_id, supported_id, to_id, agent_id, via_id, auth_id, )
        )
        r = self.simple_query(txn, 'SELECT LAST_INSERT_ID()', ())
        if r:
            id = int(r[0][0])
        else:
            id = 0
        return id

    def connect_event(self, txn, event):
        remote_ip = event['src_ip']
        sensor_id = self.get_id(txn, 'sensors', 'name', event['sensor'])
        message_id = self.get_id(txn, 'messages', 'value', event['message'])
        request_id = self.get_request_id(txn, event['headers'])

        self.simple_query(txn,
            """
            INSERT INTO `connections` (`timestamp`, `ip`, `remote_port`, `local_host`, `local_port`, `request`, `message`, `sensor`)
            VALUES (FROM_UNIXTIME(%s), %s, %s, %s, %s, %s, %s, %s)
            """,
            (event['unixtime'], remote_ip, event['src_port'], event['dst_ip'],
             event['dst_port'], request_id, message_id, sensor_id, ))

        if self.geoip:
            country, country_code, city, org, asn_num = geolocate(remote_ip, self.reader_city, self.reader_asn)
            self.simple_query(txn, """
                INSERT INTO `geolocation` (`ip`, `country_name`, `country_iso_code`, `city_name`, `org`, `org_asn`)
                VALUES (%s, %s, %s, %s, %s, %s)
                ON DUPLICATE KEY UPDATE
                    `country_name` = %s,
                    `country_iso_code` = %s,
                    `city_name` = %s,
                    `org` = %s,
                    `org_asn` = %s
                """,
                (remote_ip, country, country_code, city, org, asn_num, country, country_code, city, org, asn_num, ))




from __future__ import absolute_import

from copy import deepcopy
from json import dump
from os import linesep
from os.path import basename, dirname

from core import output
from core.tools import mkdir
from core.config import CONFIG
from core.logfile import HoneypotDailyLogFile

class Output(output.Output):

    def start(self):
        self.epoch_timestamp = CONFIG.getboolean('output_jsonlog', 'epoch_timestamp', fallback=False)
        fn = CONFIG.get('output_jsonlog', 'logfile')
        dirs = dirname(fn)
        base = basename(fn)
        mkdir(dirs)
        self.outfile = HoneypotDailyLogFile(base, dirs, defaultMode=0o664)

    def stop(self):
        self.outfile.flush()

    def write(self, event):
        if not self.epoch_timestamp:
            # We need 'unixtime' value in some other plugins
            event_dump = deepcopy(event)
            event_dump.pop('unixtime', None)
        else:
            event_dump = event
        dump(event_dump, self.outfile, separators=(',', ':'))
        self.outfile.write(linesep)
        self.outfile.flush()




from __future__ import absolute_import

from datetime import datetime

from core import output
from core.config import CONFIG

from influxdb_client import InfluxDBClient, Point, WritePrecision
from influxdb_client.client.write_api import SYNCHRONOUS

from pytz import timezone

from twisted.python.log import msg


class Output(output.Output):

    def start(self):
        host = CONFIG.get('output_influx2', 'host')
        token = CONFIG.get('output_influx2', 'token', raw=True)

        self.client = None
        try:
            self.client = InfluxDBClient(url=host, token=token)
        except Exception as e:
            msg("output_influx2: I/O error: '{}'".format(e))
            return

        if self.client is None:
            msg('output_influx2: cannot instantiate client!')
            return

        self.org = CONFIG.get('output_influx2', 'org')
        self.bucket = CONFIG.get('output_influx2', 'bucket', fallback='siphoney')
        self.write_api = self.client.write_api(write_options=SYNCHRONOUS)

    def stop(self):
        pass

    def write(self, event):
        m = Point(event['eventid'].replace('.', '_').replace('-', '_'))\
            .tag('src_ip', event['src_ip'])\
            .time(datetime.fromtimestamp(event['unixtime'], tz=timezone('UTC')), WritePrecision.NS)

        fields = [
            'src_port',
            'dst_ip',
            'dst_port',
            'eventid',
            'message',
            'sensor'
        ]
        for key in fields:
            if key in event:
                m.field(key, event[key])
        for key in event['headers']:
            m.field(key, event['headers'][key])

        self.write_api.write(bucket=self.bucket, org=self.org, record=m)




from __future__ import absolute_import

from os import linesep
from os.path import basename, dirname
from time import gmtime, strftime

from core import output
from core.config import CONFIG
from core.tools import mkdir
from core.logfile import HoneypotDailyLogFile

class Output(output.Output):

    def start(self):
        fn = CONFIG.get('output_textlog', 'logfile')
        dirs = dirname(fn)
        base = basename(fn)
        mkdir(dirs)
        self.outfile = HoneypotDailyLogFile(base, dirs, defaultMode=0o664)

    def stop(self):
        self.outfile.flush()

    def write(self, event):
        message = '[{} UTC] [SIPHoney on {}]: {} from {}:{}.{}'.format(
            strftime('%Y-%m-%d %H:%M:%S', gmtime(event['unixtime'])),
            event['sensor'], event['message'], event['src_ip'],
            event['dst_port'], linesep
        )
        self.outfile.write(message)
        self.outfile.flush()




from datetime import datetime
from ipaddress import ip_address, ip_network
from os import makedirs, path
from socket import socket, AF_INET, SOCK_DGRAM
from sys import version_info

from core.config import CONFIG

from pytz import timezone

from twisted.python.log import msg

try:
    from urllib.request import urlopen
except ImportError:
    from urllib import urlopen


if version_info[0] >= 3:
    def decode(x):
        return x.decode('utf-8', errors='ignore')
    def encode(x):
        return x.encode()
    def ord(x):
        return x
    def to_bytes(x):
        return bytes(x, 'ascii')
    def to_int(x):
        return x
    def unicode(x):
        return x
else:
    def decode(x):
        return x
    def encode(x):
        return x
    def to_bytes(x):
        return bytes(x)
    def to_int(x):
        return ord(x)


def mkdir(dir_path):
    if not dir_path:
        return
    if path.exists(dir_path) and path.isdir(dir_path):
        return
    makedirs(dir_path)


def import_plugins(cfg):
    # Load output modules (inspired by the Cowrie honeypot)
    msg('Loading the plugins...')
    output_plugins = []
    general_options = cfg
    for x in CONFIG.sections():
        if not x.startswith('output_'):
            continue
        if CONFIG.getboolean(x, 'enabled') is False:
            continue
        engine = x.split('_')[1]
        try:
            output = __import__('output_plugins.{}'.format(engine),
                                globals(), locals(), ['output'], 0).Output(general_options)
            output_plugins.append(output)
            msg('Loaded output engine: {}'.format(engine))
        except ImportError as e:
            msg('Failed to load output engine: {} due to ImportError: {}'.format(engine, e))
        except Exception as e:
            msg('Failed to load output engine: {} {}'.format(engine, e))
    return output_plugins


def stop_plugins(cfg):
    msg('Stoping the plugins...')
    for plugin in cfg['output_plugins']:
        try:
            plugin.stop()
        except Exception as e:
            msg(e)
            continue


def get_public_ip(ip_reporter):
    try:
        if version_info[0] < 3:
            return urlopen(ip_reporter).read().decode('latin1', errors='replace').encode('utf-8')
        else:
            return decode(urlopen(ip_reporter).read())
    except:
        return None


def get_local_ip():
    s = socket(AF_INET, SOCK_DGRAM)
    try:
        s.connect(('10.255.255.255', 1))
        ip = s.getsockname()[0]
    except:
        ip = '127.0.0.1'
    finally:
        s.close()
    return ip


def get_utc_time(unix_time):
    return datetime.fromtimestamp(unix_time, tz=timezone('UTC')).isoformat() + 'Z'


def write_event(event, cfg):
    ip = event['src_ip']
    for network in cfg['blacklist']:
        if ip_address(unicode(ip)) in ip_network(unicode(network)):
            return
    output_plugins = cfg['output_plugins']
    for plugin in output_plugins:
        try:
            plugin.write(event)
        except Exception as e:
            msg(e)
            continue


def geolocate(remote_ip, reader_city, reader_asn):
    try:
        response_city = reader_city.city(remote_ip)
        city = response_city.city.name
        if city is None:
            city = ''
        else:
            city = decode(city.encode('utf-8'))
        country = response_city.country.name
        if country is None:
            country = ''
            country_code = ''
        else:
            country = decode(country.encode('utf-8'))
            country_code = decode(response_city.country.iso_code.encode('utf-8'))
    except Exception as e:
        msg(e)
        city = ''
        country = ''
        country_code = ''

    try:
        response_asn = reader_asn.asn(remote_ip)
        if response_asn.autonomous_system_organization is None:
            org = ''
        else:
            org = decode(response_asn.autonomous_system_organization.encode('utf-8'))

        if response_asn.autonomous_system_number is not None:
            asn_num = response_asn.autonomous_system_number
        else:
            asn_num = 0
    except Exception as e:
        msg(e)
        org = ''
        asn_num = 0
    return country, country_code, city, org, asn_num




from configparser import ConfigParser, ExtendedInterpolation

from os import environ


def to_environ_key(key):
    return key.upper()


class EnvironmentConfigParser(ConfigParser):

    def has_option(self, section, option):
        if to_environ_key('_'.join((section, option))) in environ:
            return True
        return super(EnvironmentConfigParser, self).has_option(section, option)

    def get(self, section, option, **kwargs):
        key = to_environ_key('_'.join((section, option)))
        if key in environ:
            return environ[key]
        return super(EnvironmentConfigParser, self).get(section, option, **kwargs)


def readConfigFile(cfgfile):
    """
    Read config files and return ConfigParser object

    @param cfgfile: filename or array of filenames
    @return: ConfigParser object
    """
    parser = EnvironmentConfigParser(
        interpolation=ExtendedInterpolation(),
        converters={'list': lambda x: [i.strip() for i in x.split(',')]}
    )
    parser.read(cfgfile)
    return parser


def _config_files():
    # Import here (not at module top) to avoid any circular-import risk.
    from core.paths import workdir_path, bundled
    return [
        bundled('etc', 'honeypot.cfg.base'),	# bundled read-only defaults
        workdir_path('etc', 'honeypot.cfg'),	# site-local overrides
        workdir_path('honeypot.cfg'),		# convenience root-level override
    ]


CONFIG = readConfigFile(_config_files())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
XMPP output plugin
"""

from __future__ import absolute_import

from sys import version_info
from threading import Event, Thread
from time import gmtime, strftime

from core import output
from core.config import CONFIG

import xmpp.dispatcher
import xmpp.simplexml
import xmpp.transports
from xmpp import Client
from xmpp.protocol import JID, Message

from twisted.python.log import msg


# ---------------------------------------------------------------------------
# Python 2.7 compatibility patches for xmpppy.
#
# xmpppy 0.7.3 has two functions with Python 3-only syntax that crash
# on Python 2.7:
#
# 1. simplexml.ustr(): calls str(r, ENCODING) which is Python 3 bytes-to-str
#    syntax. On Python 2.7 the equivalent is unicode(r).
#
# 2. dispatcher.Dispatcher.Process(): calls .with_traceback() unconditionally
#    on exception objects, but that method only exists on Python 3.
#
# Both are patched below before any xmpppy code runs.
# ---------------------------------------------------------------------------
if version_info[0] < 3:
    # Patch 1: simplexml.ustr
    def _ustr(what):
        if isinstance(what, str):
            return what
        try:
            r = what.__str__()
        except AttributeError:
            r = str(what)
        if not isinstance(r, str):
            return r
        return r

    xmpp.simplexml.ustr = _ustr
    xmpp.transports.ustr = _ustr
    # xmpp.protocol also does 'from .simplexml import ustr' — patch it too
    import xmpp.protocol as _xmpp_protocol
    if hasattr(_xmpp_protocol, 'ustr'):
        _xmpp_protocol.ustr = _ustr

    # Patch 2: dispatcher.Dispatcher - must be named 'Dispatcher' so that
    # PlugIn() registers it under the right key in owner.__dict__, and must
    # define plugin/plugout explicitly so PlugIn()'s __class__.__dict__ check
    # finds them (inherited methods are not in __class__.__dict__ on Py2
    # old-style classes).
    _OriginalDispatcher = xmpp.dispatcher.Dispatcher

    class Dispatcher(_OriginalDispatcher):
        def plugin(self, owner):
            _OriginalDispatcher.plugin(self, owner)

        def plugout(self):
            _OriginalDispatcher.plugout(self)

        def Process(self, timeout=0):
            for handler in self._cycleHandlers:
                handler(self)
            if len(self._pendingExceptions) > 0:
                _pendingException = self._pendingExceptions.pop()
                ex = _pendingException[0](_pendingException[1])
                if hasattr(ex, 'with_traceback'):
                    ex = ex.with_traceback(_pendingException[2])
                raise ex
            if self._owner.Connection.pending_data(timeout):
                try:
                    data = self._owner.Connection.receive()
                except IOError:
                    return
                self.Stream.Parse(data)
                if len(self._pendingExceptions) > 0:
                    _pendingException = self._pendingExceptions.pop()
                    ex = _pendingException[0](_pendingException[1])
                    if hasattr(ex, 'with_traceback'):
                        ex = ex.with_traceback(_pendingException[2])
                    raise ex
                if data:
                    return len(data)
            return '0'

    xmpp.dispatcher.Dispatcher = Dispatcher


class Output(output.Output):
    """
    XMPP output plugin.
    Sends honeypot events as plain-text messages to a configured JID.

    xmpppy's Process(1) blocks for up to one second per call, so it runs
    in a daemon thread rather than in Twisted's reactor thread.  write()
    calls send() directly from the reactor thread; send() only writes to
    the socket while Process() only reads, so there is no write-write race.
    """

    def start(self):
        jabberid = CONFIG.get('output_xmpp', 'user')
        password = CONFIG.get('output_xmpp', 'password', raw=True)
        self.receiver = CONFIG.get('output_xmpp', 'muc')
        self._stop_event = Event()
        self._thread = None

        jid = JID(jabberid)
        self.connection = Client(server=jid.getDomain(), debug=[])

        result = self.connection.connect()
        if not result:
            msg('output_xmpp: unable to connect to {}'.format(jid.getDomain()))
            self.connection = None
            return
        msg('output_xmpp: connected ({})'.format(result))

        if not self.connection.auth(user=jid.getNode(), password=password,
                                    resource=jid.getResource()):
            msg('output_xmpp: authentication failed for {}'.format(jabberid))
            self.connection.disconnect()
            self.connection = None
            return
        msg('output_xmpp: authenticated as {}'.format(jabberid))

        self._thread = Thread(target=self._process_loop)
        self._thread.daemon = True
        self._thread.start()

    def _process_loop(self):
        while not self._stop_event.is_set():
            try:
                conn = self.connection
                if conn is None:
                    break
                if not conn.Process(1):
                    msg('output_xmpp: connection lost')
                    self.connection = None
                    break
            except Exception as e:
                msg('output_xmpp: error in Process loop: {}'.format(e))
                self.connection = None
                break

    def stop(self):
        self._stop_event.set()
        if self._thread is not None:
            self._thread.join(timeout=2)
        if self.connection is not None:
            self.connection.disconnect()
            self.connection = None

    def write(self, event):
        if self.connection is None:
            return

        message = '[{} UTC] [SIPHoney on {}]: {} from {}:{}.'.format(
            strftime('%Y-%m-%d %H:%M:%S', gmtime(event['unixtime'])),
            event['sensor'], event['message'], event['src_ip'], event['dst_port']
        )

        try:
            self.connection.send(Message(to=self.receiver, body=message))
        except Exception as e:
            msg('output_xmpp: send failed: {}'.format(e))
            self.connection = None



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import

from sys import version_info
from time import gmtime, strftime, time

from core import output
from core.config import CONFIG

from twisted.internet import reactor
from twisted.internet.task import deferLater

# --- Slack compatibility wrapper ---
if version_info[0] >= 3:
    # Python 3: slack-sdk
    from slack_sdk import WebClient as SlackClientWrapper # type: ignore
else:
    # Python 2.7: slackclient 1.x
    from slackclient import SlackClient as BaseSlackClient  # type: ignore

    class SlackResponse(object):
        """Wrap Python 2 SlackClient dict to mimic SlackResponse"""
        def __init__(self, data):
            self.data = data

    class SlackClientWrapper(object):
        def __init__(self, token):
            self._client = BaseSlackClient(token)

        def chat_postMessage(self, **kwargs):
            result = self._client.api_call("chat.postMessage", **kwargs)
            return SlackResponse(result)


class Output(output.Output):
    """
    Slack webhook output plugin.
    """

    def start(self):
        self.slack_channel = CONFIG.get('output_slack', 'channel')
        self.slack_token = CONFIG.get('output_slack', 'token')
        self.delay = CONFIG.getfloat('output_slack', 'delay', fallback=1.2)
        self.sc = SlackClientWrapper(self.slack_token)
        self.last_sent = 0
        self.requests_list = []

    def stop(self):
        pass

    def write(self, event):
        message = '[{} UTC] [SIPHoney on {}]: {} from {}:{}.'.format(
            strftime('%Y-%m-%d %H:%M:%S', gmtime(event['unixtime'])),
            event['sensor'], event['message'], event['src_ip'], event['dst_port']
        )
        self.requests_list.append(message)
        self._drain()

    def _drain(self):
        """
        Send the next queued message if the rate-limit window has elapsed.
        If messages remain, schedule another drain after self.delay seconds.
        """
        if not self.requests_list:
            return
        now = time()
        elapsed = now - self.last_sent
        if elapsed < self.delay:
            # Re-schedule after the remainder of the delay window
            deferLater(reactor, self.delay - elapsed, self._drain)
            return
        self.last_sent = now
        message = self.requests_list.pop(0)
        self.sc.chat_postMessage(channel=self.slack_channel, text=message)
        if self.requests_list:
            deferLater(reactor, self.delay, self._drain)



"""
Simple Datadog HTTP logger.
"""

from __future__ import absolute_import

from io import BytesIO
from json import dumps
from platform import node

from core import output
from core.config import CONFIG
from core.tools import to_bytes

from twisted.internet import reactor
from twisted.python.log import msg
from twisted.web import client, http_headers
from twisted.web.client import FileBodyProducer
from twisted.internet.ssl import ClientContextFactory


class WebClientContextFactory(ClientContextFactory):
    def getContext(self, hostname, port):
        return ClientContextFactory.getContext(self)


class QuietHTTP11ClientFactory(client._HTTP11ClientFactory):
    noisy = False


class Output(output.Output):
    def start(self):
        self.url = CONFIG.get('output_datadog', 'url')
        self.api_key = CONFIG.get('output_datadog', 'api_key', fallback='')
        if len(self.api_key) == 0:
            msg('output_datadog: API key is not defined.')
        self.ddsource = CONFIG.get('output_datadog', 'ddsource', fallback='siphoney')
        self.ddtags = CONFIG.get('output_datadog', 'ddtags', fallback='env:dev')
        self.service = CONFIG.get('output_datadog', 'service', fallback='honeypot')
        self.hostname = CONFIG.get('output_datadog', 'hostname', fallback=node())

        contextFactory = WebClientContextFactory()
        myQuietPool = client.HTTPConnectionPool(reactor)
        myQuietPool._factory = QuietHTTP11ClientFactory
        self.agent = client.Agent(reactor, contextFactory=contextFactory, pool=myQuietPool)

    def stop(self):
        pass

    def write(self, event):
        messg = '{} INFO [SIP Honey on {}]: {} from {}:{}.'.format(
            event['timestamp'], event['sensor'],
            event['message'], event['src_ip'], event['dst_port']
        )
        event['message'] = messg
        event['ddsource'] = self.ddsource
        event['ddtags'] = self.ddtags
        event['hostname'] = self.hostname
        event['service'] = self.service
        self.postentry(event)

    def postentry(self, entry):
        base_headers = {
            b'Accept': [b'application/json'],
            b'Content-Type': [b'application/json'],
            b'DD-API-KEY': [to_bytes(self.api_key)],
        }
        headers = http_headers.Headers(base_headers)
        body = FileBodyProducer(BytesIO(to_bytes(dumps(entry, sort_keys=True))))
        self.agent.request(b'POST', to_bytes(self.url), headers, body)





from __future__ import absolute_import

from json import dumps
from socket import socket, AF_INET, SOCK_STREAM

from core import output
from core.config import CONFIG
from core.tools import encode


class Output(output.Output):
    """
    socketlog output
    """

    def start(self):
        timeout = CONFIG.getint('output_socketlog', 'timeout')
        addr = CONFIG.get('output_socketlog', 'address')
        host = addr.split(':')[0]
        port = int(addr.split(':')[1])

        self.sock = socket(AF_INET, SOCK_STREAM)
        self.sock.settimeout(timeout)
        self.sock.connect((host, port))

    def stop(self):
        self.sock.close()

    def write(self, event):
        message = dumps(event, sort_keys=True) + '\n'

        try:
            self.sock.sendall(encode(message))
        except OSError as ex:
            if ex.errno == 32:  # Broken pipe
                self.start()
                self.sock.sendall(encode(message))
            else:
                raise




from __future__ import absolute_import

import syslog

from core import output
from core.config import CONFIG


def formatCef(logentry):
    """
    Take logentry and turn into CEF string
    """
    # Jan 18 11:07:53 host CEF:Version|Device Vendor|Device Product|
    # Device Version|Signature ID|Name|Severity|[Extension]
    cefVendor = 'Bontchev'
    cefProduct = 'SIPHoney'
    cefVersion = '1.0'
    cefSignature = logentry['eventid']
    cefName = logentry['eventid']
    cefSeverity = '5'

    cefExtensions = {
        'deviceExternalId': logentry['sensor'],
        'src': logentry['src_ip'],
        'spt': logentry['src_port'],
        'dpt': logentry['dst_port'],
        'dst': logentry['dst_ip'],
        'msg': logentry['message'],
        'proto': 'tcp'
    }

    cefList = []
    for key in list(cefExtensions.keys()):
        value = str(cefExtensions[key])
        cefList.append('{}={}'.format(key, value))

    cefExtension = ' '.join(cefList)

    cefString = 'CEF:0|' + \
                cefVendor + '|' + \
                cefProduct + '|' + \
                cefVersion + '|' + \
                cefSignature + '|' + \
                cefName + '|' + \
                cefSeverity + '|' + \
                cefExtension

    return cefString


class Output(output.Output):

    def start(self):
        facilityString = CONFIG.get('output_localsyslog', 'facility', fallback='USER')
        facility = vars(syslog)['LOG_' + facilityString]
        syslog.openlog(logoption=syslog.LOG_PID, facility=facility)

    def stop(self):
        pass

    def write(self, event):
        syslog.syslog(syslog.LOG_INFO, formatCef(event))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Simple Telegram Bot logger

from __future__ import absolute_import

from json import loads
from time import gmtime, strftime, time

from core import output
from core.config import CONFIG
from core.tools import decode, encode

try:
    # Python 3
    from html import escape
    from urllib.parse import urlencode
except ImportError:
    # Python 2
    from cgi import escape
    from urllib import urlencode

from twisted.internet import reactor
from twisted.internet.ssl import ClientContextFactory
from twisted.internet.task import deferLater
from twisted.python.log import msg
from twisted.web.client import (
    Agent,
    HTTPConnectionPool,
    _HTTP11ClientFactory,
    readBody
)
from twisted.web.http_headers import Headers


class WebClientContextFactory(ClientContextFactory):

    def getContext(self, hostname, port):
        return ClientContextFactory.getContext(self)


class QuietHTTP11ClientFactory(_HTTP11ClientFactory):
    noisy = False


class Output(output.Output):
    """
    Telegram bot output plugin.
    """

    def start(self):
        self.bot_token = CONFIG.get('output_telegram', 'bot_token')
        self.chat_id = CONFIG.get('output_telegram', 'chat_id')
        self.delay = CONFIG.getfloat('output_telegram', 'delay', fallback=2.0)

        contextFactory = WebClientContextFactory()
        pool = HTTPConnectionPool(reactor)
        pool._factory = QuietHTTP11ClientFactory
        self.agent = Agent(reactor, contextFactory=contextFactory, pool=pool)
        self.last_sent = 0
        self.requests_list = []

    def stop(self):
        pass

    def write(self, event):
        message = '[{} UTC] [SIPHoney on {}]: {} from {}:{}.'.format(
            strftime('%Y-%m-%d %H:%M:%S', gmtime(event['unixtime'])),
            event['sensor'],
            event['message'],
            event['src_ip'],
            event['dst_port']
        )
        self.requests_list.append(message)
        self._drain()

    def _drain(self):
        """Send the next queued message if the rate-limit window has elapsed.
        If messages remain, schedule another drain after self.delay seconds."""
        if not self.requests_list:
            return
        now = time()
        elapsed = now - self.last_sent
        if elapsed < self.delay:
            deferLater(reactor, self.delay - elapsed, self._drain)
            return
        self.last_sent = now
        message = self.requests_list.pop(0)
        d = self._send(message)
        if self.requests_list:
            deferLater(reactor, self.delay, self._drain)
        return d

    def _send(self, message):

        def cbBody(body):
            return processResult(body)

        def cbPartial(failure):
            """
            Google HTTP Server does not set Content-Length. Twisted marks it as partial
            """
            failure.printTraceback()
            return processResult(failure.value.response)

        def cbResponse(response):
            if response.code in [200, 201]:
                return
            msg('Telegram error: {} {}'.format(response.code, decode(response.phrase)))
            d = readBody(response)
            d.addCallback(cbBody)
            d.addErrback(cbPartial)
            return d

        def cbError(failure):
            failure.printTraceback()

        def processResult(result):
            try:
                j = loads(result)
                msg('Telegram response: {}'.format(j.get('description', '')))
            except Exception:
                pass

        headers = Headers({
            b'User-Agent': [b'SIPHoney']
        })
        escaped_text = escape(message)
        params = {
            'chat_id': self.chat_id,
            'parse_mode': 'HTML',
            'text': escaped_text
        }
        url = 'https://api.telegram.org/bot{}/sendMessage?{}'.format(self.bot_token, urlencode(params))
        d = self.agent.request(b'GET', encode(url), headers, None)
        d.addCallback(cbResponse)
        d.addErrback(cbError)
        return d




from __future__ import absolute_import

from core import output
from core.config import CONFIG
from core.tools import geolocate

from warnings import filterwarnings
from cryptography.utils import CryptographyDeprecationWarning
filterwarnings('ignore', category=CryptographyDeprecationWarning)

from pymongo import MongoClient
from pymongo.server_api import ServerApi
from geoip2.database import Reader

from twisted.python.log import msg


class Output(output.Output):

    def insert_one(self, collection, event):
        try:
            return collection.insert_one(event).inserted_id
        except Exception as e:
            msg('output_mongodb: Error: {}'.format(e))
            return None

    def start(self):
        host = CONFIG.get('output_mongodb', 'host', fallback='localhost')
        username = CONFIG.get('output_mongodb', 'username', fallback='', raw=True)
        password = CONFIG.get('output_mongodb', 'password', fallback='', raw=True)
        db_name = CONFIG.get('output_mongodb', 'database', fallback='siphoney')
        db_addr = CONFIG.get('output_mongodb', 'connection_string')
        db_addr = db_addr.format(username, password, host)

        self.geoip = CONFIG.getboolean('output_mongodb', 'geoip', fallback=True)

        try:
            self.mongo_client = MongoClient(db_addr, server_api=ServerApi('1'))
            self.mongo_db = self.mongo_client[db_name]
            self.col_events = self.mongo_db['connections']
            if self.geoip:
                self.col_geolocation = self.mongo_db['geolocation']
        except Exception as e:
            msg('output_mongodb: Error: {}'.format(e))

        if self.geoip:
            geoipdb_city_path = CONFIG.get('output_mongodb', 'geoip_citydb', fallback='data/GeoLite2-City.mmdb')
            geoipdb_asn_path = CONFIG.get('output_mongodb', 'geoip_asndb', fallback='data/GeoLite2-ASN.mmdb')
            try:
                self.reader_city = Reader(geoipdb_city_path)
            except Exception:
                self.reader_city = None
                msg('Failed to open City GeoIP database {}'.format(geoipdb_city_path))
            try:
                self.reader_asn = Reader(geoipdb_asn_path)
            except Exception:
                self.reader_asn = None
                msg('Failed to open ASN GeoIP database {}'.format(geoipdb_asn_path))

    def stop(self):
        self.mongo_client.close()
        if self.geoip:
            if self.reader_city is not None:
                self.reader_city.close()
            if self.reader_asn is not None:
                self.reader_asn.close()

    def write(self, event):
        self.insert_one(self.col_events, event)
        if self.geoip:
            remote_ip = event['src_ip']
            if not self.col_geolocation.find_one({'ip': remote_ip}):
                country, country_code, city, org, asn_num = geolocate(remote_ip, self.reader_city, self.reader_asn)
                geo_entry = {
                    'ip': remote_ip,
                    'country': country,
                    'country_code': country_code,
                    'city': city,
                    'org': org,
                    'asn': asn_num
                }
                self.insert_one(self.col_geolocation, geo_entry)




from __future__ import absolute_import

from core import output
from core.config import CONFIG
from core.tools import geolocate

from geoip2.database import Reader

from psycopg2 import OperationalError

from twisted.enterprise.adbapi import ConnectionPool
from twisted.python.log import msg


class Output(output.Output):

    def start(self):
        host = CONFIG.get('output_postgres', 'host', fallback='localhost')
        port = CONFIG.getint('output_postgres', 'port', fallback=5432)
        username = CONFIG.get('output_postgres', 'username', fallback='siphoney')
        password = CONFIG.get('output_postgres', 'password')
        database = CONFIG.get('output_postgres', 'database', fallback='siphoney')
        self.debug = CONFIG.getboolean('output_postgres', 'debug', fallback=False)
        self.geoip = CONFIG.getboolean('output_postgres', 'geoip', fallback=True)

        try:
            self.dbh = ConnectionPool(
                'psycopg2',
                database=database,
                user=username,
                password=password,
                host=host,
                port=port,
                cp_min=1,
                cp_max=1
            )
        except OperationalError as e:
            msg('output_postgres: PostgreSQL Error: {}'.format(e))

        if self.geoip:
            geoipdb_city_path = CONFIG.get('output_postgres', 'geoip_citydb', fallback='data/GeoLite2-City.mmdb')
            geoipdb_asn_path = CONFIG.get('output_postgres', 'geoip_asndb', fallback='data/GeoLite2-ASN.mmdb')
            try:
                self.reader_city = Reader(geoipdb_city_path)
            except Exception:
                self.reader_city = None
                msg('Failed to open City GeoIP database {}'.format(geoipdb_city_path))
            try:
                self.reader_asn = Reader(geoipdb_asn_path)
            except Exception:
                self.reader_asn = None
                msg('Failed to open ASN GeoIP database {}'.format(geoipdb_asn_path))

    def stop(self):
        if self.geoip:
            if self.reader_city is not None:
                self.reader_city.close()
            if self.reader_asn is not None:
                self.reader_asn.close()

    def write(self, event):
        self.dbh.runInteraction(self.connect_event, event)

    def simple_query(self, txn, sql, args, returns_value=True):
        if self.debug:
            msg('output_postgres: postgres query: {} {}'.format(sql, repr(args)))
        result = None
        try:
            txn.execute(sql, args)
            if returns_value:
                result = txn.fetchone()
        except Exception as e:
            msg('output_postgres: postgres Error: {}'.format(e))
        return result

    def get_id(self, txn, table, column, entry):
        r = self.simple_query(
            txn,
            "SELECT id FROM {} WHERE {} = %s".format(table, column),
            (entry, )
        )
        if r:
            id = int(r[0])
        else:
            r = self.simple_query(
                txn,
                "INSERT INTO {} ({}) VALUES (%s) RETURNING id".format(table, column),
                (entry, )
            )
            if r:
                id = int(r[0])
            else:
                id = 0
        return id

    def get_request_id(self, txn, headers):
        accept_id = allow_id = callids_id = contact_id = conlen_id = contype_id = cseq_id = expires_id = from_id = forward_id = supported_id = to_id = agent_id = via_id = auth_id = 0
        for key, value in headers.items():
            if key == 'accept':
                accept_id = self.get_id(txn, 'accepts', 'val', value)
            elif key == 'allow':
                allow_id = self.get_id(txn, 'allows', 'val', value)
            elif key == 'call-id':
                callids_id = self.get_id(txn, 'callids', 'val', value)
            elif key == 'contact':
                contact_id = self.get_id(txn, 'contacts', 'val', value)
            elif key == 'content-length':
                conlen_id = self.get_id(txn, 'con_lens', 'val', value)
            elif key == 'content-type':
                contype_id = self.get_id(txn, 'con_types', 'val', value)
            elif key == 'cseq':
                cseq_id = self.get_id(txn, 'cseqs', 'val', value)
            elif key == 'expires':
                expires_id = self.get_id(txn, 'expires', 'val', value)
            elif key == 'from':
                from_id = self.get_id(txn, 'froms', 'val', value)
            elif key == 'max-forwards':
                forward_id = self.get_id(txn, 'forwards', 'val', value)
            elif key == 'supported':
                supported_id = self.get_id(txn, 'supporteds', 'val', value)
            elif key == 'to':
                to_id = self.get_id(txn, 'tos', 'val', value)
            elif key == 'user-agent':
                agent_id = self.get_id(txn, 'agents', 'val', value)
            elif key == 'via':
                via_id = self.get_id(txn, 'vias', 'val', value)
            elif key == 'authorization':
                auth_id = self.get_id(txn, 'authorizations', 'val', value)
            else:
                self.get_id(txn, 'unknown', 'header', key)
        r = self.simple_query(
            txn,
            """
            INSERT INTO requests (accept, allow, callid, contact, con_len, con_type, cseq, expires, frm, forwards, supportd, dest, agent, via, auth)
            VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)
            RETURNING id
            """,
            (accept_id, allow_id, callids_id, contact_id, conlen_id, contype_id, cseq_id, expires_id, from_id, forward_id, supported_id, to_id, agent_id, via_id, auth_id, )
        )
        if r:
            id = int(r[0])
        else:
            id = 0
        return id

    def connect_event(self, txn, event):
        remote_ip = event['src_ip']
        sensor_id = self.get_id(txn, 'sensors', 'sname', event['sensor'])
        message_id = self.get_id(txn, 'messages', 'val', event['message'])
        request_id = self.get_request_id(txn, event['headers'])

        self.simple_query(
            txn,
            """
            INSERT INTO connections (time_stamp, ip, remote_port, local_host, local_port, request, messg, sensor)
            VALUES (to_timestamp(%s), %s, %s, %s, %s, %s, %s, %s)
            """,
            (event['unixtime'], remote_ip, event['src_port'], event['dst_ip'],
             event['dst_port'], request_id, message_id, sensor_id, ),
            False
        )

        if self.geoip:
            country, country_code, city, org, asn_num = geolocate(remote_ip, self.reader_city, self.reader_asn)
            self.simple_query(
                txn,
                """
                INSERT INTO geolocation (ip, country_name, country_iso_code, city_name, org, org_asn)
                VALUES (%s, %s, %s, %s, %s, %s)
                ON CONFLICT (ip) DO UPDATE SET
                    country_name = EXCLUDED.country_name,
                    country_iso_code = EXCLUDED.country_iso_code,
                    city_name = EXCLUDED.city_name,
                    org = EXCLUDED.org,
                    org_asn = EXCLUDED.org_asn
                """,
                (remote_ip, country, country_code, city, org, asn_num, ),
                False
            )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function

__description__ = 'Test the SIPHoney honeypot'
__license__ = 'GPL'
__VERSION__ = '1.0.0'
__author__ = 'Vesselin Bontchev'
__email__ = 'vbontchev@yahoo.com'


from argparse import ArgumentParser
from contextlib import contextmanager
from socket import AF_INET, IPPROTO_UDP, SOCK_DGRAM, socket, timeout
from sys import version_info


if version_info[0] >= 3:
    def decode(x):
        return x.decode('utf-8', errors='ignore')
    def encode(x):
        return x.encode()
else:
    def decode(x):
        return x
    def encode(x):
        return x


@contextmanager
def connect_to(host, port):
    connection = None
    try:
        connection = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)
        connection.connect((host, int(port)))
        yield connection
    finally:
        if connection:
            connection.close()


def get_options():
    parser = ArgumentParser(description=__description__)

    parser.add_argument('-v', '--version', action='version', version='%(prog)s version ' + __VERSION__)
    parser.add_argument('-P', '--port', type=int, default=5060,
                        help='Port to send data to (default: %(default)d)')
    parser.add_argument('-H', '--host', default='127.0.0.1',
                        help='Host to communicate with (default: %(default)s)')
    parser.add_argument('-s', '--sender', default='"sipvicious"<sip:100@1.1.1.1>',
                        help='From address (default: %(default)s)')
    parser.add_argument('-t', '--to', default='<sip:user_2@test.test>',
                        help='To address (default: %(default)s)')
    parser.add_argument('-c', '--callid', default='1@0.0.0.0',
                        help='Call ID (default: %(default)s)')
    parser.add_argument('-r', '--request', default='OPTIONS',
                        help='Request (default: %(default)s)')
    args = parser.parse_args()
    return args


def main():
    args = get_options()

    with connect_to(args.host, args.port) as connection:
        payload = (
            '{} sip:user_1@test.test SIP/2.0\r\n'
            'To: {}\r\n'
            'From: {};tag=none\r\n'
            'Call-ID: {}\r\n'
            'CSeq: 1 INVITE\r\n'
            'Contact: {}\r\n'
            'Via: SIP/2.0/TCP 0.0.0.0;branch=34uiddhjczqw3mq23\r\n'
            'Content-Length: 1\r\n'
            '\r\n'
            'T'
        ).format(args.request.upper(), args.to, args.sender, args.callid, args.sender)
        connection.send(encode(payload))
        connection.settimeout(1)
        print('Response:\n')
        while True:
            try:
                data = connection.recv(2048)
                if data:
                    print(decode(data.strip().replace(b'\r\n', b'\n')))
            except timeout:
                break
            except Exception as e:
                print(e)
                break
    print()


if __name__ == '__main__':
    main()




from __future__ import absolute_import

from core import output
from core.config import CONFIG
from core.tools import geolocate

from couchdb import Server
from geoip2.database import Reader

from twisted.python.log import msg

class Output(output.Output):

    def start(self):
        host = CONFIG.get('output_couch', 'host', fallback='localhost')
        port = CONFIG.getint('output_couch', 'port', fallback=5984)
        username = CONFIG.get('output_couch', 'username', fallback='siphoney', raw=True)
        password = CONFIG.get('output_couch', 'password', fallback='', raw=True)
        db_name = CONFIG.get('output_couch', 'database', fallback='siphoney')

        try:
            couchserver = Server('http://{}:{}@{}:{}'.format(username, password, host, port))

            if db_name in couchserver:
                self.couch_db = couchserver[db_name]
            else:
                self.couch_db = couchserver.create(db_name)
        except Exception as e:
            msg('output_couch: Error: {}'.format(e))

        self.geoip = CONFIG.getboolean('output_couch', 'geoip', fallback=True)

        if self.geoip:
            geoipdb_city_path = CONFIG.get('output_couch', 'geoip_citydb', fallback='data/GeoLite2-City.mmdb')
            geoipdb_asn_path = CONFIG.get('output_couch', 'geoip_asndb', fallback='data/GeoLite2-ASN.mmdb')
            try:
                self.reader_city = Reader(geoipdb_city_path)
            except:
                self.reader_city = None
                msg('Failed to open City GeoIP database {}'.format(geoipdb_city_path))

            try:
                self.reader_asn = Reader(geoipdb_asn_path)
            except:
                self.reader_asn = None
                msg('Failed to open ASN GeoIP database {}'.format(geoipdb_asn_path))

    def stop(self):
        if self.geoip:
            if self.reader_city is not None:
                self.reader_city.close()
            if self.reader_asn is not None:
                self.reader_asn.close()

    def write(self, event):
        if self.geoip:
            country, country_code, city, org, asn_num = geolocate(event['src_ip'], self.reader_city, self.reader_asn)
            event['country'] = country
            event['country_code'] = country_code
            event['city'] = city
            event['org'] = org
            event['asn'] = asn_num

        try:
            self.couch_db.save(event)
        except Exception as e:
            msg('output_couch: Error: {}'.format(e))




from sys import stdout
from datetime import datetime

from pytz import timezone

from twisted.python import log, util
from twisted.python.logfile import DailyLogFile


class HoneypotDailyLogFile(DailyLogFile):
    """
    Overload original Twisted with improved date formatting
    """

    def suffix(self, tupledate):
        """
        Return the suffix given a (year, month, day) tuple or unixtime
        """
        try:
            return "{:02d}-{:02d}-{:02d}".format(tupledate[0], tupledate[1], tupledate[2])
        except Exception:
            # try taking a float unixtime
            return '_'.join(map(str, self.toDate(tupledate)))


def myFLOemit(self, eventDict):
    """
    Format the given log event as text and write it to the output file.

    @param eventDict: a log event
    @type eventDict: L{dict} mapping L{str} (native string) to L{object}
    """

    # Custom emit for FileLogObserver
    text = log.textFromEventDict(eventDict)
    if text is None:
        return
    timeStr = self.formatTime(eventDict['time'])
    fmtDict = {
        'text': text.replace('\n', '\n\t')
    }
    msgStr = log._safeFormat('%(text)s\n', fmtDict)
    util.untilConcludes(self.write, timeStr + ' ' + msgStr)
    util.untilConcludes(self.flush)


def myFLOformatTime(self, when):
    """
    Log time in UTC

    By default it's formatted as an ISO8601-like string (ISO8601 date and
    ISO8601 time separated by a space). It can be customized using the
    C{timeFormat} attribute, which will be used as input for the underlying
    L{datetime.datetime.strftime} call.

    @type when: C{int}
    @param when: POSIX (ie, UTC) timestamp.

    @rtype: C{str}
    """
    timeFormatString = self.timeFormat
    if timeFormatString is None:
        timeFormatString = '[%Y-%m-%d %H:%M:%S.%fZ]'
    return datetime.fromtimestamp(when, tz=timezone('UTC')).strftime(timeFormatString)


def set_logger(cfg_options):
    log.FileLogObserver.emit = myFLOemit
    log.FileLogObserver.formatTime = myFLOformatTime
    if cfg_options['logfile'] is None:
        log.startLogging(stdout)
    else:
        log.startLogging(HoneypotDailyLogFile.fromFullPath(cfg_options['logfile']), setStdout=False)




from __future__ import absolute_import

from json import dumps

from core import output
from core.config import CONFIG
from core.tools import to_bytes

from confluent_kafka import Producer

from twisted.python.log import msg


class Output(output.Output):

    def start(self):
        self.topic = CONFIG.get('output_kafka', 'topic', fallback='siphoney')
        self.debug = CONFIG.getboolean('output_kafka', 'debug', fallback=False)

        self.producer = Producer({
            'bootstrap.servers': '{}:{}'.format(
                CONFIG.get('output_kafka', 'host', fallback='localhost'),
                CONFIG.get('output_kafka', 'port', fallback=9092)
            ),
            #'sasl.mechanism': 'SCRAM-SHA-256',
            #'security.protocol': 'SASL_SSL',
            #'sasl.username': CONFIG.get('output_kafka', 'username'),
            #'sasl.password': CONFIG.get('output_kafka', 'password')
        })

    def stop(self):
        self.producer.flush()

    def write(self, event):
        self.postentry(event)

    def delivery_callback(self, err, message):
        if err:
            msg('output_kafka: Message failed delivery: {}.'.format(err))
        else:
            if self.debug:
                msg('output_kafka: Message delivered to {} [{}] @ {}'.format(
                    message.topic(), message.partition(), message.offset()
                ))

    def postentry(self, event):
        try:
            self.producer.produce(
                self.topic,
                to_bytes(dumps(event, sort_keys=True)),
                callback=self.delivery_callback
            )
            self.producer.poll(0)
            self.producer.flush()
        except Exception as e:
            msg('Kafka error: {}'.format(e))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import

from json import dumps

from core import output
from core.config import CONFIG

from requests import post

from twisted.internet.threads import deferToThread
from twisted.python.log import msg

class Output(output.Output):

    def start(self):
        self.debug = CONFIG.getboolean('output_redisdb', 'debug', fallback=False)
        host = CONFIG.get('output_redisdb', 'host')
        self.password = CONFIG.get('output_redisdb', 'password')
        keyname = CONFIG.get('output_redisdb', 'keyname', fallback='siphoney')
        self.url = 'https://{}/lpush/{}'.format(host, keyname)

    def stop(self):
        pass

    def write(self, event):
        payload = dumps(event, sort_keys=True)
        headers = {
            b'Authorization': 'Bearer {}'.format(self.password),
            b'Content-Type': b'application/json'
        }

        if self.debug:
            msg(payload)

        d = deferToThread(post, self.url, data=payload, headers=headers, timeout=5)
        d.addCallback(self._on_response)
        d.addErrback(self._on_error)

    def _on_response(self, response):
        if self.debug and response.status_code != 200:
            msg('[REST] Response code: {}'.format(response.status_code))

    def _on_error(self, failure):
        msg('[REST] REST error: {}'.format(failure))



"""
paths.py - Single source of truth for runtime path resolution.

The honeypot needs a "working directory" containing:
  etc/          config file (honeypot.cfg.base)
  log/          rotating log files (created on demand)
  data/         geolocation databases, SQLite db, etc.

Priority for locating the working directory:
  1. SIPHONEY_WORKDIR environment variable
  2. Current working directory

The bundled read-only default (`etc/honeypot.cfg.base`)
is installed inside the `siphoney` package and located via the package's
own __file__ attribute, which works on all Python versions without
requiring pkg_resources or importlib.resources.
"""

from __future__ import absolute_import


from os import getcwd, environ
from os.path import abspath, dirname, join


def get_workdir():
    """Return the absolute path to the runtime working directory."""
    env = environ.get('SIPHONEY_WORKDIR', '').strip()
    if env:
        return abspath(env)
    return getcwd()


def workdir_path(*parts):
    """Return an absolute path rooted at the working directory."""
    return join(get_workdir(), *parts)


def bundled(*parts):
    """
    Return the filesystem path to a file bundled inside the installed package.
    Arguments are path components relative to the siphoney/data/ directory,
    passed as separate strings (like os.path.join) to avoid hardcoded separators.

    Uses the package's own __file__ to locate the data directory, which works
    on all Python versions (2.7+) without requiring pkg_resources or
    importlib.resources.
    """
    # siphoney/data/ lives alongside this module's package (core/ is a sibling
    # of siphoney/), so we go up one level from core/ to find siphoney/data/.
    here = dirname(abspath(__file__))
    package_dir = join(dirname(here), 'siphoney')
    return join(package_dir, 'data', *parts)



# siphoney: the installable Python package for the siphoney honeypot.
#
# After `pip install siphoney`, use the `siphoney` command:
#
#   siphoney init    -- scaffold a working directory
#   siphoney run     -- start the honeypot in the foreground
#   siphoney start   -- start the honeypot in the background
#   siphoney stop    -- stop the background honeypot
#   siphoney status  -- show running status
#
# NOTE: siphoney/honeypot.py is generated at build time by the build_py
# hook in setup.py - it is a copy of the top-level honeypot.py bundled so
# that the `siphoney run/start` entry point can locate and run it.
# It is listed in .gitignore and should not be committed.
#
# Contents:
#   cli.py        the `siphoney` console entry point (init/run/start/stop/status)
#   honeypot.py   the main honeypot script (copied from repo root at build time)
#   data/         bundled read-only assets copied to the working directory
#                 by `siphoney init`:
#     etc/        default configuration templates
#     responses/  MongoDB wire-protocol response stubs
#     docs/       documentation and SQL schema files
#     test/       test.py for verifying a running honeypot



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Simple Discord webhook logger
"""

from __future__ import absolute_import

from io import BytesIO
from json import dumps, loads
from time import gmtime, strftime, time

from core import output
from core.config import CONFIG
from core.tools import decode, to_bytes

from twisted.internet import reactor
from twisted.internet.ssl import ClientContextFactory
from twisted.internet.task import deferLater
from twisted.python.log import msg
from twisted.web.client import (
    Agent,
    FileBodyProducer,
    HTTPConnectionPool,
    _HTTP11ClientFactory,
    readBody
)
from twisted.web.http_headers import Headers


class WebClientContextFactory(ClientContextFactory):
    def getContext(self, hostname, port):
        return ClientContextFactory.getContext(self)


class QuietHTTP11ClientFactory(_HTTP11ClientFactory):
    noisy = False


class Output(output.Output):

    def start(self):
        self.url = to_bytes(CONFIG.get('output_discord', 'url'))
        self.delay = CONFIG.getfloat('output_discord', 'delay', fallback=2.0)
        contextFactory = WebClientContextFactory()
        pool = HTTPConnectionPool(reactor)
        pool._factory = QuietHTTP11ClientFactory
        self.agent = Agent(reactor, contextFactory=contextFactory, pool=pool)
        self.last_sent = 0
        self.requests_list = []

    def stop(self):
        pass

    def write(self, event):
        message = '__New event__\n[{} UTC] [SIPHoney on {}]: {} from {}:{}.\n'.format(
            strftime('%Y-%m-%d %H:%M:%S', gmtime(event['unixtime'])),
            event['sensor'], event['message'], event['src_ip'], event['dst_port']
        )
        self.requests_list.append({'content': message})
        self._drain()

    def _drain(self):
        """
        Send the next queued entry if the rate-limit window has elapsed.
        If entries remain, schedule another drain after self.delay seconds.
        """
        if not self.requests_list:
            return
        now = time()
        elapsed = now - self.last_sent
        if elapsed < self.delay:
            deferLater(reactor, self.delay - elapsed, self._drain)
            return
        self.last_sent = now
        entry = self.requests_list.pop(0)
        d = self._post(entry)
        if self.requests_list:
            deferLater(reactor, self.delay, self._drain)
        return d

    def _post(self, entry):

        def cbBody(body):
            return processResult(body)

        def cbPartial(failure):
            """
            Google HTTP Server does not set Content-Length. Twisted marks it as partial
            """
            failure.printTraceback()
            return processResult(failure.value)

        def cbResponse(response):
            if response is None or response.code in [200, 201, 204]:
                return
            msg('Discord error: {} {}'.format(response.code, decode(response.phrase)))
            d = readBody(response)
            d.addCallback(cbBody)
            d.addErrback(cbPartial)
            return d

        def cbError(failure):
            failure.printTraceback()

        def processResult(result):
            if result:
                try:
                    j = loads(result)
                    msg('Discord response: {}'.format(j.get('message', '')))
                except Exception:
                    pass

        headers = Headers({b'Content-Type': [b'application/json']})
        body = FileBodyProducer(BytesIO(to_bytes(dumps(entry, sort_keys=True))))
        d = self.agent.request(b'POST', self.url, headers, body)
        d.addCallback(cbResponse)
        d.addErrback(cbError)
        return d




from __future__ import absolute_import

from time import sleep

from core import output
from core.config import CONFIG

from rethinkdb import r

from twisted.python.log import msg


class Output(output.Output):

    def start(self):
        host = CONFIG.get('output_rethinkdblog', 'host', fallback='localhost')
        port = CONFIG.getint('output_rethinkdblog', 'port', fallback=28015)
        self.db = CONFIG.get('output_rethinkdblog', 'db', fallback='siphoney')
        password = CONFIG.get('output_rethinkdblog', 'password', raw=True)
        self.table = CONFIG.get('output_rethinkdblog', 'table', fallback='events')

        try:
            self.connection = r.connect(host=host, port=port, password=password)

            if self.db not in r.db_list().run(self.connection):
                r.db_create(self.db).run(self.connection)
            for _ in range(50):  # ~5 seconds max
                if self.db in r.db_list().run(self.connection):
                    break
                sleep(0.1)
            if self.table not in r.db(self.db).table_list().run(self.connection):
                r.db(self.db).table_create(self.table).run(self.connection)

        except r.RqlError as err:
            msg('output_rethinkdblog: Error: {}'.format(err.message))

    def stop(self):
        if getattr(self, 'connection', None):
            self.connection.close()

    def write(self, event):
        try:
            r.db(self.db).table(self.table).insert(event).run(self.connection)
        except r.RqlRuntimeError as err:
            msg('output_rethinkdblog: Error: {}'.format(err.message))



#!/usr/bin/env python
# -*- coding: utf-8 -*-


"""
siphoney command-line entry point.

After `pip install siphoney` the user gets a `siphoney` command:

  siphoney init [DIR]   -- scaffold a working directory (default: cwd)
  siphoney run          -- start the honeypot in the foreground
  siphoney start        -- start in the background (Linux, macOS, Windows)
  siphoney stop         -- stop a backgrounded honeypot
  siphoney restart      -- restart (stop and start in the backgrounded) the honeypot
  siphoney status       -- show running status

The working directory is resolved in this order:
  1. DIR argument (for init) or --workdir flag (for other commands)
  2. SIPHONEY_WORKDIR environment variable
  3. Current working directory
"""

from __future__ import absolute_import, print_function

from argparse import ArgumentParser
from importlib import import_module
from os import (
    chdir,
    devnull,
    dup2,
    environ,
    kill,
    makedirs,
    name,
    remove,
    walk,
    O_RDWR,
)
from os import open as _open, chmod
from os.path import (
    abspath,
    dirname,
    exists,
    isdir,
    join,
    relpath,
)
from re import search, IGNORECASE
from shutil import copy2
from stat import S_IXUSR, S_IXGRP, S_IXOTH
import sys

from core.paths import bundled, get_workdir, workdir_path


# ---------------------------------------------------------------------------
# helpers
# ---------------------------------------------------------------------------

def _ensure_dir(p):
    if not isdir(p):
        makedirs(p)


def _file_differs(a, b):
    with open(a, 'r') as fa, open(b, 'r') as fb:
        return fa.read() != fb.read()


def _copy_if_missing(src, dst, label=None):
    # Guard against copying a file onto itself (e.g. in a source checkout).
    if abspath(src) == abspath(dst):
        return False
    if not exists(dst) or _file_differs(src, dst):
        copy2(src, dst)
        if label is not None:
            print('  copied  {}'.format(label))
        return True
    if label is not None:
        print('  exists  {}'.format(label))
    return False


def _set_workdir(path):
    environ['SIPHONEY_WORKDIR'] = abspath(path)


def _pidfile():
    return workdir_path('siphoney.pid')


def _pid_running(pid):
    if name == 'nt':
        # os.kill(pid, 0) is a Unix idiom that doesn't work on Windows -
        # it raises SystemError. Use OpenProcess instead: a non-NULL handle
        # means the process exists; NULL means it doesn't (or we lack
        # permission, but that also means it's not ours to manage).
        import ctypes
        SYNCHRONIZE = 0x00100000
        handle = ctypes.windll.kernel32.OpenProcess(SYNCHRONIZE, False, pid)
        if not handle:
            return False
        ctypes.windll.kernel32.CloseHandle(handle)
        return True
    try:
        kill(pid, 0)
        return True
    except OSError:
        return False


def _locate_honeypot_script():
    """Return the absolute path to honeypot.py inside the installed package."""
    # honeypot.py is bundled inside the siphoney package directory alongside
    # this file (cli.py). Using __file__ avoids any dependency on pkg_resources
    # or importlib.resources and works on Python 2.7+.
    here = dirname(abspath(__file__))
    script_path = join(here, 'honeypot.py')
    if not exists(script_path):
        raise RuntimeError(
            'Cannot locate honeypot.py. '
            'Expected inside the siphoney package directory: {}'.format(here)
        )
    return script_path


def _run_main():
    """
    Import and run honeypot.main().

    honeypot.py lives inside the siphoney package directory. We import it
    directly using importlib so it runs as a proper module rather than a
    top-level script, avoiding any sys.path manipulation.
    """
    script_path = _locate_honeypot_script()
    script_dir = dirname(script_path)

    # Ensure the package directory is on sys.path so that honeypot.py's own
    # imports (from core.xxx, from output_plugins.xxx, etc.) resolve correctly.
    if script_dir not in sys.path:
        sys.path.insert(0, script_dir)

    # Import as 'honeypot'; clear any stale cached version first.
    if 'honeypot' in sys.modules:
        del sys.modules['honeypot']
    mod = import_module('honeypot')
    mod.main()


# ---------------------------------------------------------------------------
# init
# ---------------------------------------------------------------------------

def cmd_init(args):

    target = abspath(args.dir)
    _set_workdir(target)
    print('Initialising siphoney working directory: {}'.format(target))

    # Runtime directories
    for d in ('etc', 'log', 'data'):
        _ensure_dir(join(target, d))
        print('  mkdir  {}'.format(d))

    # Config templates
    for fname in ('honeypot.cfg.base',):
        src = bundled('etc', fname)
        dst = join(target, 'etc', fname)
        _copy_if_missing(src, dst, join('etc', fname))

    # docs/ tree - walk the entire bundled docs directory and copy everything.
    docs_src = bundled('docs')
    docs_dst = join(target, 'docs')
    for dirpath, dirnames, filenames in walk(docs_src):
        rel_dir = relpath(dirpath, docs_src)
        dst_dir = join(docs_dst, rel_dir) if rel_dir != '.' else docs_dst
        _ensure_dir(dst_dir)
        for fname in filenames:
            # geoipupdtask.ps1 is handled separately below (Windows only)
            if fname == 'geoipupdtask.ps1':
                continue
            src = join(dirpath, fname)
            dst = join(dst_dir, fname)
            rel = join('docs', rel_dir, fname) if rel_dir != '.' else join('docs', fname)
            _copy_if_missing(src, dst, rel)

    _copy_if_missing(bundled('test', 'baseline'), join(target, 'docs', 'baseline'), join('docs', 'baseline'))
    _copy_if_missing(bundled('test', 'test.py'), join(target, 'test.py'), 'test.py')
    if name == 'nt':
        _copy_if_missing(bundled('test', 'test.ps1'), join(target, 'test.ps1'), 'test.ps1')
    else:
        _copy_if_missing(bundled('test', 'test.sh'), join(target, 'test.sh'), 'test.sh')
        chmod(join(target, 'test.sh', S_IXUSR | S_IXGRP | S_IXOTH))
    _dockerfile = bundled('Dockerfile')
    if exists(_dockerfile):
        _copy_if_missing(_dockerfile, join(target, 'Dockerfile'), 'Dockerfile')
    # geoipupdtask.ps1 is a Windows Task Scheduler helper - copy on Windows only
    if name == 'nt':
        geo_src = bundled('docs', 'geoipupdtask.ps1')
        if exists(geo_src):
            _copy_if_missing(geo_src, join(target, 'geoipupdtask.ps1'), 'geoipupdtask.ps1')

    print()
    print('Next steps:')
    print('  1. Edit {} with your settings.'.format(join(target, 'etc', 'honeypot.cfg')))
    print('  2. Place GeoLite2-City.mmdb and GeoLite2-ASN.mmdb in {}.'.format(join(target, 'data')))
    print('  3. Run:  siphoney run   (foreground)')
    print('     or:   siphoney start (background)')


# ---------------------------------------------------------------------------
# run (foreground)
# ---------------------------------------------------------------------------

def cmd_run(args):
    # honeypot.py has its own argparse parser and will see sys.argv directly.
    # Keep only arguments that come after 'run' so honeypot.py's parser sees
    # only the options intended for it (e.g. siphoney run --port 5060).
    try:
        run_idx = sys.argv.index('run')
        sys.argv = [sys.argv[0]] + sys.argv[run_idx + 1:]
    except ValueError:
        sys.argv = [sys.argv[0]]

    # Output plugins use relative paths from CONFIG (e.g. 'log/siphoney.json')
    # without going through workdir_path(). Changing to the working directory
    # before starting ensures these paths resolve correctly.
    chdir(get_workdir())

    _run_main()


# ---------------------------------------------------------------------------
# start (background) -- cross-platform
# ---------------------------------------------------------------------------

def cmd_start(args):
    pidfile = _pidfile()

    # Check if already running
    if exists(pidfile):
        with open(pidfile) as fh:
            pid_str = fh.read().strip()
        if pid_str:
            try:
                pid = int(pid_str)
                if _pid_running(pid):
                    print('The honeypot is already running (PID {}).'.format(pid))
                    return
            except ValueError:
                pass

    print('Starting the honeypot...')
    if name == 'nt':
        _start_windows(pidfile)
    else:
        _start_posix(pidfile)


def _start_posix(pidfile):
    """Fork to background on POSIX (Linux, macOS)."""
    from os import fork, setsid
    pid = fork()
    if pid > 0:
        # Parent: record child PID and exit
        with open(pidfile, 'w') as fh:
            fh.write(str(pid))
        print('The honeypot was started successfully (PID {}).'.format(pid))
        return

    # Child: detach from terminal
    setsid()
    _devnull = _open(devnull, O_RDWR)
    for fd in (0, 1, 2):
        dup2(_devnull, fd)

    # Clear the 'start' subcommand from sys.argv before honeypot.py's parser runs
    try:
        start_idx = sys.argv.index('start')
        sys.argv = [sys.argv[0]] + sys.argv[start_idx + 1:]
    except ValueError:
        sys.argv = [sys.argv[0]]

    # Ensure relative paths in plugin configs resolve against the working directory
    chdir(get_workdir())

    _run_main()


def _start_windows(pidfile):
    """
    Spawn a detached, hidden background process on Windows.

    Uses pythonw.exe (the windowless interpreter) to avoid any console window.
    DETACHED_PROCESS + CREATE_NO_WINDOW ensure the child survives parent exit
    and has no console of its own.
    """
    from subprocess import Popen, STARTF_USESHOWWINDOW, STARTUPINFO

    # Defined as literals for Python 2.7/3.6 compatibility - these constants
    # were only added to subprocess as named attributes in Python 3.7.
    DETACHED_PROCESS = 0x00000008
    CREATE_NO_WINDOW = 0x08000000

    startupinfo = STARTUPINFO()
    startupinfo.dwFlags    |= STARTF_USESHOWWINDOW
    startupinfo.wShowWindow = 0  # SW_HIDE

    # pythonw.exe is the windowless interpreter Windows ships for background
    # scripts; more reliable than CREATE_NO_WINDOW for Twisted-based processes.
    pythonw = join(dirname(sys.executable), 'pythonw.exe')
    interpreter = pythonw if exists(pythonw) else sys.executable
    cmd = [interpreter, '-m', 'siphoney.cli', 'run']

    _devnull = open(devnull, 'wb')
    try:
        proc = Popen(
            cmd,
            creationflags=DETACHED_PROCESS | CREATE_NO_WINDOW,
            startupinfo=startupinfo,
            stdin=_devnull,
            stdout=_devnull,
            stderr=_devnull,
            env=environ.copy(),
        )
    finally:
        _devnull.close()

    with open(pidfile, 'w') as fh:
        fh.write(str(proc.pid))
    print('The honeypot was started successfully (PID {}).'.format(proc.pid))


# ---------------------------------------------------------------------------
# stop
# ---------------------------------------------------------------------------

def cmd_stop(args):
    pidfile = _pidfile()
    if not exists(pidfile):
        print('The honeypot is not running (no PID file).')
        return

    with open(pidfile) as fh:
        pid_str = fh.read().strip()

    if not pid_str:
        remove(pidfile)
        print('Empty PID file removed.')
        return

    try:
        pid = int(pid_str)
    except ValueError:
        remove(pidfile)
        print('Corrupt PID file removed.')
        return

    if not _pid_running(pid):
        remove(pidfile)
        print('Stale PID file (PID {} not running) removed.'.format(pid))
        return

    print('Stopping the honeypot (PID {})... '.format(pid), end='')

    if name == 'nt':
        _stop_windows(pid, pidfile)
    else:
        _stop_posix(pid, pidfile)


def _stop_posix(pid, pidfile):
    from signal import SIGKILL, SIGTERM
    from time import sleep
    kill(pid, SIGTERM)
    for _ in range(60):
        sleep(1)
        if not _pid_running(pid):
            print('Stopped.')
            remove(pidfile)
            return
    print()
    print('Did not stop gracefully; sending SIGKILL.')
    kill(pid, SIGKILL)
    remove(pidfile)


def _stop_windows(pid, pidfile):
    """
    Terminate the background honeypot process on Windows.

    The process was launched with pythonw.exe + CREATE_NO_WINDOW so it has no
    console. GenerateConsoleCtrlEvent only works for processes sharing a
    console or in the same process group - neither applies here. We therefore
    go straight to TerminateProcess via taskkill /F.
    """
    from subprocess import call

    _devnull = open(devnull, 'wb')
    try:
        ret = call(
            ['taskkill', '/F', '/PID', str(pid)],
            stdout=_devnull, stderr=_devnull,
        )
    finally:
        _devnull.close()

    if ret == 0 or not _pid_running(pid):
        print('Stopped.')
    else:
        print()
        print('Warning: taskkill returned {}, process may still be running.'.format(ret))
    remove(pidfile)


# ---------------------------------------------------------------------------
# restart
# ---------------------------------------------------------------------------

def cmd_restart(args):
    cmd_stop(args)
    cmd_start(args)


# ---------------------------------------------------------------------------
# status
# ---------------------------------------------------------------------------

def cmd_status(args):
    pidfile = _pidfile()
    if not exists(pidfile):
        print('The honeypot is not running.')
        return
    with open(pidfile) as fh:
        pid_str = fh.read().strip()
    if not pid_str:
        remove(pidfile)
        print('The honeypot is not running (empty PID file removed).')
        return
    try:
        pid = int(pid_str)
    except ValueError:
        remove(pidfile)
        print('Corrupt PID file removed.')
        return
    if _pid_running(pid):
        print('The honeypot is running (PID: {}).'.format(pid))
    else:
        remove(pidfile)
        print('The honeypot is not running (stale PID file for PID {} removed).'.format(pid))


# ---------------------------------------------------------------------------
# entry point
# ---------------------------------------------------------------------------

def main():
    parser = ArgumentParser(
        prog='siphoney',
        description='siphoney - A SIP Honeypot',
    )

    # Read version from honeypot.py without importing it (avoids Twisted import
    # at startup) and without using pkg_resources (deprecated in setuptools 81+).
    try:
        with open(_locate_honeypot_script()) as fh:
            _content = fh.read()
        _m = search(r"__VERSION__\s*=\s*['\"]([^'\"]+)['\"]", _content, IGNORECASE)
        _version = _m.group(1) if _m else 'unknown'
    except Exception:
        _version = 'unknown'

    parser.add_argument('-v', '--version', action='version', version=_version)
    parser.add_argument(
        '-w', '--workdir', default=None,
        help='Working directory (overrides SIPHONEY_WORKDIR and cwd)'
    )

    sub = parser.add_subparsers(dest='command', metavar='COMMAND')
    sub.required = True

    p_init = sub.add_parser('init', help='Scaffold a working directory')
    p_init.add_argument(
        'dir', nargs='?', default='.',
        help='Directory to initialize (default: current directory)'
    )
    p_init.set_defaults(func=cmd_init)

    p_run = sub.add_parser('run', help='Start the honeypot in the foreground')
    p_run.set_defaults(func=cmd_run)

    p_start = sub.add_parser('start', help='Start the honeypot in the background')
    p_start.set_defaults(func=cmd_start)

    sub.add_parser('stop', help='Stop the backgrounded honeypot').set_defaults(func=cmd_stop)

    p_restart = sub.add_parser(
        'restart', help='Restart (stop and start) the honeypot in the background'
    )
    p_restart.set_defaults(func=cmd_restart)

    for _p in (p_run, p_start, p_restart):
        _p.add_argument('-p', '--port', type=int, help='Port to listen on (default: 5060)')
        _p.add_argument('-l', '--logfile', help='Log file (default: stdout)')
        _p.add_argument('-s', '--sensor', help='Sensor name (default: hostname)')

    sub.add_parser('status',  help='Show running status').set_defaults(func=cmd_status)

    args = parser.parse_args()

    if args.workdir:
        _set_workdir(args.workdir)

    args.func(args)


if __name__ == '__main__':
    main()




from __future__ import absolute_import

from sys import version_info

from core import output
from core.config import CONFIG

from twisted.python.log import msg

PY2 = version_info[0] < 3

if PY2:
    import elasticsearch    # type: ignore
    from elasticsearch import Elasticsearch, NotFoundError  # type: ignore
    version_module = elasticsearch
    version_attr = '__version__'
else:
    import elasticsearch8
    from elasticsearch8 import Elasticsearch, NotFoundError
    version_module = elasticsearch8
    version_attr = '__versionstr__'


class Output(output.Output):

    def start(self):
        host = CONFIG.get('output_elastic', 'host', fallback='localhost')
        port = CONFIG.getint('output_elastic', 'port', fallback=9200)
        username = CONFIG.get('output_elastic', 'username', fallback=None)
        password = CONFIG.get('output_elastic', 'password', fallback=None)
        use_ssl = CONFIG.getboolean('output_elastic', 'ssl', fallback=False)
        verify_certs = CONFIG.getboolean('output_elastic', 'verify_certs', fallback=False)
        ca_certs = CONFIG.get('output_elastic', 'ca_certs', fallback=None)
        self.index = CONFIG.get('output_elastic', 'index', fallback='siphoney')
        self.pipeline = CONFIG.get('output_elastic', 'pipeline', fallback='geoip')

        scheme = 'https' if use_ssl else 'http'
        hosts = [{'host': host, 'port': port, 'scheme': scheme}]
        kwargs = {}
        if username and password:
            kwargs['http_auth' if PY2 else 'basic_auth'] = (username, password)
        if use_ssl:
            kwargs['verify_certs'] = verify_certs
            if verify_certs and ca_certs:
                kwargs['ca_certs'] = ca_certs

        # Create client
        self.es = Elasticsearch(hosts=hosts, **kwargs)

        # Detect major version from module attribute
        ver_attr_value = getattr(version_module, version_attr)
        if isinstance(ver_attr_value, tuple):
            # Python 2.7 elasticsearch 7.x
            self.es_ver = ver_attr_value[0]
        else:
            # Python 3.6+ elasticsearch8
            self.es_ver = int(str(ver_attr_value).split('.')[0])

        # Test connection
        try:
            info = self.es.info()
            msg('output_elastic: connected to ES version {}'.format(info['version']['number']))
        except Exception as e:
            msg('output_elastic: connection failed: {}'.format(repr(e)))
            raise

        # Ensure index exists
        if not self.es.indices.exists(index=self.index):
            self.es.indices.create(index=self.index)

        # Setup geoip
        if self.pipeline == 'geoip':
            self._setup_geoip_mapping()
            self._setup_geoip_pipeline()

    def _setup_geoip_mapping(self):
        try:
            body = {
                'properties': {
                    'geo': {
                        'properties': {
                            'location': {
                                'type': 'geo_point'
                            }
                        }
                    }
                }
            }
            if self.es_ver >= 8:
                self.es.indices.put_mapping(index=self.index, properties=body['properties'])
            else:
                self.es.indices.put_mapping(index=self.index, body=body)
        except Exception as e:
            msg('output_elastic: mapping setup failed: {}'.format(repr(e)))

    def _setup_geoip_pipeline(self):
        try:
            self.es.ingest.get_pipeline(id=self.pipeline)
        except NotFoundError:
            body = {
                'description': 'Add geoip info',
                'processors': [
                    {
                        'geoip': {
                            'field': 'src_ip',
                            'target_field': 'geo',
                            'database_file': 'GeoLite2-City.mmdb'
                        }
                    }
                ]
            }
            try:
                if self.es_ver >= 8:
                    self.es.ingest.put_pipeline(
                        id=self.pipeline,
                        processors=body['processors'],
                        description=body['description']
                    )
                else:
                    self.es.ingest.put_pipeline(id=self.pipeline, body=body)
            except Exception as e:
                msg('output_elastic: pipeline creation failed: {}'.format(repr(e)))

    def stop(self):
        pass

    def write(self, entry):
        try:
            if self.es_ver >= 8:
                self.es.index(index=self.index, document=entry, pipeline=self.pipeline)
            else:
                self.es.index(index=self.index, body=entry, pipeline=self.pipeline)
        except Exception as e:
            msg('output_elastic: write failed: {}'.format(repr(e)))
            if hasattr(e, 'info'):
                msg('output_elastic info: {}'.format(e.info))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import


from io import open
from os import makedirs, sep, walk
from os.path import (
    abspath,
    dirname,
    exists,
    isdir,
    join,
    relpath,
)
from re import search, sub, IGNORECASE
from setuptools import setup
from setuptools.command.build_py import build_py as _build_py
from setuptools.command.egg_info import egg_info as _egg_info
from shutil import copy2, rmtree
from wheel.bdist_wheel import bdist_wheel as _bdist_wheel


REPO_BASE = 'https://gitlab.com/bontchev/siphoney/-/blob/master'

try:
    class bdist_wheel(_bdist_wheel):
        """
        Remove the egg-info directory after building the wheel.
        pip 19.x (Python 2.7) mistakes a leftover egg-info in the source tree
        for an existing installation and skips both copying files to
        site-packages and generating the entry point script.
        """
        def run(self):
            _bdist_wheel.run(self)
            here = dirname(abspath(__file__))
            egg_info = join(here, 'siphoney.egg-info')
            if isdir(egg_info):
                rmtree(egg_info)
    _have_wheel = True
except ImportError:
    _have_wheel = False


class egg_info(_egg_info):
    """
    Redirect the egg-info directory into build/ instead of the source root.

    When a user runs 'python setup.py bdist_wheel' the default behaviour is to
    write siphoney.egg-info into the source tree. pip2 (19.x) then finds this
    egg-info and mistakenly treats the source directory as an existing
    installation, causing 'pip2 install dist/siphoney-*.whl' to silently skip
    installing to site-packages and skip generating the console entry point.

    Redirecting to build/ avoids this: build/ is cleaned between builds and
    pip never looks there for installed packages.
    """
    def initialize_options(self):
        _egg_info.initialize_options(self)
        here = dirname(abspath(__file__))
        build_dir = join(here, 'build')
        if not isdir(build_dir):
            makedirs(build_dir)
        self.egg_base = build_dir


class build_py(_build_py):
    """
    Sync honeypot.py from the repo root into siphoney/ before building.
    This keeps a single source of truth (the root-level script) while ensuring
    the installed package contains it for use by the entry point.

    Note: 'python -m build' uses an isolated PEP 517 environment. The sdist
    already contains honeypot.py at the root, so when the wheel is built from
    the sdist the copy is present and this hook finds it correctly.
    """
    def run(self):
        here = dirname(abspath(__file__))
        src = join(here, 'honeypot.py')
        dst = join(here, 'siphoney', 'honeypot.py')
        if exists(src):
            copy2(src, dst)
        # Also copy Dockerfile from repo root into siphoney/data/ so that
        # `siphoney init` can place it in the working directory.
        dockerfile_src = join(here, 'Dockerfile')
        dockerfile_dst = join(here, 'siphoney', 'data', 'Dockerfile')
        if exists(dockerfile_src):
            copy2(dockerfile_src, dockerfile_dst)
        plugindoc_src = join(here, 'output_plugins', 'README.md')
        plugindoc_dst = join(here, 'siphoney', 'data', 'docs', 'PLUGINS.md')
        if exists(plugindoc_src):
            copy2(plugindoc_src, plugindoc_dst)
        _build_py.run(self)


def get_package_data():
    """
    Walk siphoney/ and return every non-.py file as a package_data entry.
    Files under data/ are always included even if they have a .py extension
    (e.g. data/test/test.py). Using os.walk means new files are picked up
    automatically without needing to update this list manually.
    """
    here = dirname(abspath(__file__))
    pkg_root = join(here, 'siphoney')
    result = []
    for dirpath, dirnames, filenames in walk(pkg_root):
        dirnames[:] = [d for d in dirnames
                       if d not in ('__pycache__', 'siphoney.egg-info')]
        for filename in filenames:
            abs_path = join(dirpath, filename)
            rel_path = relpath(abs_path, pkg_root)
            is_under_data = rel_path.startswith('data' + sep) or rel_path == 'data'
            # Regular .py package modules are handled by packages=; skip them.
            # Exception: honeypot.py lives directly in siphoney/ and is the
            # main script, included here so the entry point can import it.
            if filename.endswith('.py') and not is_under_data and filename != 'honeypot.py':
                continue
            # Dockerfile is copied into siphoney/data/ by the build hook;
            # include it explicitly even if it isn't present in the source tree.
            result.append(rel_path.replace(sep, '/'))
    # Always include these hook-generated files, whether present yet or not.
    for extra in ('honeypot.py', join('data', 'Dockerfile')):
        entry = extra.replace(sep, '/')
        if entry not in result:
            result.append(entry)
    return result


def get_version():
    here = dirname(abspath(__file__))
    with open(join(here, 'honeypot.py')) as fh:
        content = fh.read()
    match = search(r"__VERSION__\s*=\s*['\"]([^'\"]+)['\"]", content, IGNORECASE)
    if not match:
        raise RuntimeError('Cannot find __VERSION__ in honeypot.py')
    return match.group(1)


# Copy hook-generated files into the package directory before setup() runs,
# so they are present when setuptools assembles the wheel's file list.
# This is necessary because get_package_data() is called at setup() time,
# before any build hooks run, so the files must already exist on disk.
_here = dirname(abspath(__file__))
for _src, _dst in [
    (join(_here, 'honeypot.py'), join(_here, 'siphoney', 'honeypot.py')),
    (join(_here, 'Dockerfile'),  join(_here, 'siphoney', 'data', 'Dockerfile')),
    (join(_here, 'output_plugins', 'README.md'), join(_here, 'siphoney', 'data', 'docs', 'PLUGINS.md')),
]:
    if exists(_src):
        copy2(_src, _dst)


def make_pypi_readme():
    with open('README.md', encoding='utf-8') as fh:
        content = fh.read()
    # Rewrite markdown links: [text](relative/path) -> [text](REPO_BASE/relative/path)
    def absolutify(m):
        url = m.group(2)
        if url.startswith('http://') or url.startswith('https://') or url.startswith('#'):
            return m.group(0)  # already absolute or anchor, leave alone
        return '[{}]({}/{})'.format(m.group(1), REPO_BASE, url)
    return sub(r'\[([^\]]+)\]\(([^)]+)\)', absolutify, content)


extras={
    'couchdb': ['couchdb'],
    'datadog': [
        'cryptography<=2.8 ; python_version < "3"',
        'pyOpenSSL<=18.0.0 ; python_version < "3"',
        'cryptography ; python_version >= "3"',
        'pyOpenSSL ; python_version >= "3"',
    ],
    'discord': [],
    'elastic': [
        'elasticsearch<=7.13 ; python_version < "3"',
        'numpy<=1.16.6 ; python_version < "3"',
        'elasticsearch8>=8.12.0,<9.0.0 ; python_version >= "3"',
        'numpy ; python_version >= "3"',
    ],
    'hpfeed': [
        'Automat<20 ; python_version < "3"',
        'hpfeeds>=3.0.0',
    ],
    'influx2': ['influxdb-client ; python_version >= "3"'],  # Python 3 only
    'jsonlog': [],
    'kafka': [
        'confluent-kafka<1.0 ; python_version < "3"',
        'confluent-kafka ; python_version >= "3"',
    ],
    'localsyslog': [],
    'mongodb': [
        'pymongo<=3.13.0 ; python_version < "3"',
        'dnspython ; python_version < "3"',
        'pymongo ; python_version >= "3"',
    ],
    'mysql': [
        'PyMySQL ; python_version < "3"',
        'mysqlclient>=1.3.12 ; python_version >= "3"',
    ],
    'nlcvapi': [
        'pyOpenSSL<=18.0.0 ; python_version < "3"',
        'pyOpenSSL ; python_version >= "3"',
    ],
    'postgres': ['psycopg2-binary'],
    'redisdb': [
        'redis<=3.5.3 ; python_version < "3"',
        'redis ; python_version >= "3"',
    ],
    'rethinkdblog': [
        'rethinkdb>=2.4',
        'looseversion',
    ],
    'slack': [
        'slackclient<3 ; python_version < "3"',
        'slack-sdk ; python_version >= "3"',
    ],
    'socketlog': [],
    'sqlite': [],
    'telegram': [],
    'textlog': [],
    'xmpp': ['xmpppy>=0.7.3'],
}

extras['all'] = sorted({
    dep
    for name, deps in extras.items()
    if name != 'all'
    for dep in deps
})

setup(
    name='siphoney',
    version=get_version(),
    description='A SIP Honeypot',
    long_description=make_pypi_readme(),
    long_description_content_type='text/markdown',
    author='Vesselin Bontchev',
    author_email='vbontchev@yahoo.com',
    url='https://gitlab.com/bontchev/siphoney',
    license='GPL-3.0-only',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Science/Research',
        'Intended Audience :: System Administrators',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3.13',
        'Programming Language :: Python :: 3.14',
        'Topic :: Security',
    ],
    python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*, !=3.5.*',

    # -------------------------------------------------------------------------
    # Installed packages:
    #   - siphoney       the console entry point + all bundled data assets
    #   - core           the honeypot engine (config, protocol, tools, ...)
    #   - output_plugins one module per output backend
    # -------------------------------------------------------------------------
    packages=['siphoney', 'core', 'output_plugins'],
    package_data={
        'siphoney': get_package_data(),
    },

    cmdclass=dict(
        {'build_py': build_py},
        **({'bdist_wheel': bdist_wheel} if _have_wheel else {})
    ),

    entry_points={
        'console_scripts': [
            'siphoney = siphoney.cli:main',
        ],
    },

    install_requires=[
        'configparser>=3.5.0',
        'geoip2>=2.7.0',
        'ipaddress ; python_version < "3"',
        'maxminddb>=1.3.0',
        'pytz',
        'requests<=2.27.1 ; python_version < "3"',
        'requests ; python_version >= "3"',
        'service_identity<=18.1.0 ; python_version < "3"',
        'service_identity ; python_version >= "3"',
        'twisted>=20.3.0,<21 ; python_version < "3"',
        'twisted>=21 ; python_version >= "3"',
    ],

    extras_require = extras
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import

from io import BytesIO
from json import dumps, loads

from core import output
from core.config import CONFIG
from core.tools import decode, geolocate, to_bytes

from geoip2.database import Reader

from twisted.internet import reactor
from twisted.internet.ssl import ClientContextFactory
from twisted.python.log import msg
from twisted.web.client import (
    Agent,
    FileBodyProducer,
    HTTPConnectionPool,
    _HTTP11ClientFactory,
    readBody,
)
from twisted.web.http_headers import Headers


class WebClientContextFactory(ClientContextFactory):

    def getContext(self, hostname, port):
        return ClientContextFactory.getContext(self)


class QuietHTTP11ClientFactory(_HTTP11ClientFactory):
    noisy = False


class Output(output.Output):

    def start(self):
        self.host = CONFIG.get('output_nlcvapi', 'host', fallback='https://api.nlcv.bas.bg/v1.0/honeypot')
        self.geoip = CONFIG.getboolean('output_nlcvapi', 'geoip', fallback=True)
        contextFactory = WebClientContextFactory()
        pool = HTTPConnectionPool(reactor)
        pool._factory = QuietHTTP11ClientFactory
        self.agent = Agent(reactor, contextFactory=contextFactory, pool=pool)

        if self.geoip:
            geoipdb_city_path = CONFIG.get('output_nlcvapi', 'geoip_citydb', fallback='data/GeoLite2-City.mmdb')
            geoipdb_asn_path = CONFIG.get('output_nlcvapi', 'geoip_asndb', fallback='data/GeoLite2-ASN.mmdb')
            try:
                self.reader_city = Reader(geoipdb_city_path)
            except Exception:
                self.reader_city = None
                msg('Failed to open City GeoIP database {}'.format(geoipdb_city_path))
            try:
                self.reader_asn = Reader(geoipdb_asn_path)
            except Exception:
                self.reader_asn = None
                msg('Failed to open ASN GeoIP database {}'.format(geoipdb_asn_path))

    def stop(self):
        if self.geoip:
            if getattr(self, 'reader_city', None) is not None:
                self.reader_city.close()
            if getattr(self, 'reader_asn', None) is not None:
                self.reader_asn.close()

    def write(self, event):
        event['honeypot'] = 'siphoney'
        if self.geoip:
            country, country_code, city, org, asn_num = geolocate(event['src_ip'], self.reader_city, self.reader_asn)
            event['country'] = country
            event['country_code'] = country_code
            event['city'] = city
            event['org'] = org
            event['asn'] = asn_num
        self._postentry(event)

    def _postentry(self, entry):

        def cbBody(body):
            return processResult(body)

        def cbPartial(failure):
            """
            Google HTTP Server does not set Content-Length. Twisted marks it as partial
            """
            failure.printTraceback()
            return processResult(failure.value.response)

        def cbResponse(response):
            if response.code in (200, 201):
                return
            msg('NLCVAPI response: {}: "{}"'.format(response.code, decode(response.phrase)))
            d = readBody(response)
            d.addCallback(cbBody)
            d.addErrback(cbPartial)
            return d

        def cbError(failure):
            failure.printTraceback()

        def processResult(result):
            try:
                j = loads(result)
                msg('NLCVAPI response: {}'.format(j.get('message', '')))
            except Exception:
                pass

        headers = Headers({
            b'User-Agent':   [b'SIPHoney'],
            b'Content-Type': [b'application/json'],
        })
        body = FileBodyProducer(BytesIO(to_bytes(dumps(entry, sort_keys=True))))
        d = self.agent.request(b'POST', to_bytes(self.host), headers, body)
        d.addCallback(cbResponse)
        d.addErrback(cbError)
        return d



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from argparse import ArgumentParser
from ipaddress import ip_network
from os.path import join
from socket import gethostname
from sys import version_info

from core.config import CONFIG
from core.logfile import set_logger
from core.protocol import SIPServer
from core.tools import (
    get_public_ip,
    import_plugins,
    mkdir,
    stop_plugins,
)

from twisted.internet import error
from twisted.internet.reactor import listenUDP, run
from twisted.python.log import msg


__VERSION__ = '2.0.1'
__description__ = 'A SIP Honeypot'
__license__ = 'GPLv3'
__uri__ = 'https://gitlab.com/bontchev/siphoney'
__author__ = 'Vesselin Bontchev'
__email__ = 'vbontchev@yahoo.com'


if version_info[0] >= 3:
    def unicode(x):
        return x


def get_options(cfg_options):
    parser = ArgumentParser(description=__description__)

    parser.add_argument('-v', '--version', action='version', version=__VERSION__)
    parser.add_argument('-p', '--port', type=int, default=cfg_options['port'],
                        help='Port to listen on (default: %(default)s)')
    parser.add_argument('-l', '--logfile', type=str, default=cfg_options['logfile'],
                        help='Log file (default: stdout)')
    parser.add_argument('-s', '--sensor', type=str, default=cfg_options['sensor'],
                        help='Sensor name (default: %(default)s)')
    args = parser.parse_args()
    return args


def set_options():
    cfg_options = {}

    cfg_options['port'] = CONFIG.getint('honeypot', 'listen_port', fallback=5060)
    log_name = CONFIG.get('honeypot', 'log_filename', fallback='')
    if log_name:
        logdir = CONFIG.get('honeypot', 'log_path', fallback='log')
        mkdir(logdir)
        cfg_options['logfile'] = join(logdir, log_name)
    else:
        cfg_options['logfile'] = None
    cfg_options['sensor'] = CONFIG.get('honeypot', 'sensor_name', fallback=gethostname())

    args = get_options(cfg_options)

    cfg_options['port'] = args.port
    cfg_options['logfile'] = args.logfile
    cfg_options['sensor'] = args.sensor
    cfg_options['public_ip_url'] = CONFIG.get('honeypot', 'public_ip_url', fallback='https://ident.me')
    cfg_options['report_public_ip'] = CONFIG.getboolean('honeypot', 'report_public_ip', fallback=False)

    pub_ip = get_public_ip(cfg_options['public_ip_url'])
    if pub_ip is None:
        cfg_options['report_public_ip'] = False
        cfg_options['public_ip'] = '127.0.0.1'
    else:
        cfg_options['public_ip'] = pub_ip

    cfg_options['blacklist'] = CONFIG.get('honeypot', 'blacklist', fallback='127.0.0.1,192.168.0.0/16').split(',')

    return cfg_options


def main():
    cfg_options = set_options()

    set_logger(cfg_options)

    msg(__description__ + ' by ' + __author__)

    blacklist = []
    for network in cfg_options['blacklist']:
        try:
            if network:
                ip_network(unicode(network))
                blacklist += [network]
        except ValueError:
            msg('Blacklist element "{}" is not a valid IP address; ignored.'.format(network))
    cfg_options['blacklist'] = blacklist

    cfg_options['output_plugins'] = import_plugins(cfg_options)

    try:
        listenUDP(port=cfg_options['port'], protocol=SIPServer(cfg_options))
    except error.CannotListenError as e:
        msg('Cannot listen on port {}: {}'.format(e.port, e.socketError.strerror))
        stop_plugins(cfg_options)
        return
    run()
    msg('Shutdown requested, exiting...')
    stop_plugins(cfg_options)


if __name__ == '__main__':
    main()




from __future__ import absolute_import


from ipaddress import ip_address, ip_network
from sys import version_info
from time import time

from core.tools import (
    get_local_ip,
    get_utc_time,
    write_event
)

from twisted.protocols.sip import Base, MessagesParser
from twisted.python.log import msg


if version_info[0] >= 3:
    def unicode(x):
        return x


class SIPServer(Base):
    def __init__(self, options):
        self.cfg = options
        self.messages = []
        self.parser = MessagesParser(self.addMessage)

    def handle_request(self, message, addr):
        headers = {}
        for item, value in message.headers.items():
            headers.update({item: ','.join(value)})

        self.report_event(addr, str(message), headers)

        request = message.method
        if request not in [
            'INVITE', 'ACK', 'BYE', 'CANCEL', 'REGISTER',
            'OPTIONS', 'PRACK', 'SUBSCRIBE', 'NOTIFY',
            'PUBLISH', 'INFO', 'REFER', 'MESSAGE', 'UPDATE'
        ]:
            response = self.responseFromRequest(501, message)
        elif request in ['INVITE', 'REGISTER']:
            response = self.responseFromRequest(401, message)
            response.addHeader(
                'WWW-Authenticate',
                'Digest algorithm=MD5, realm="asterisk", nonce="36aa22cf"'
            )
        elif request == 'SUBSCRIBE':
            response = self.responseFromRequest(403, message)
        elif request in ['NOTIFY', 'PUBLISH']:
            response = self.responseFromRequest(405, message)
            # Twisted doesn't support SIP error 489, do it manually
            response.code = 489
            response.phrase = 'Bad Event'
        elif request == 'REFER':
            response = self.responseFromRequest(603, message)
        elif request == 'MESSAGE':
            response = self.responseFromRequest(415, message)
        elif request == 'OPTIONS':
            response = self.responseFromRequest(200, message)
        elif request == 'ACK':
            return
        else:
            response = self.responseFromRequest(481, message)
        response.addHeader('Server', 'Asterisk PBX 16.28.0')
        response.addHeader('Allow', 'INVITE, ACK, CANCEL, OPTIONS, BYE, REFER, SUBSCRIBE, NOTIFY, INFO, PUBLISH, MESSAGE')
        response.addHeader('Supported', 'replaces, timer')
        response.addHeader('Accept', 'application/sdp')
        response.addHeader('Content-Length', 0)
        response.creationFinished()
        self.deliverResponse(response)

    def report_event(self, addr, message, headers):
        ip = addr[0]
        for network in self.cfg['blacklist']:
            if ip_address(unicode(ip)) in ip_network(unicode(network)):
                return
        unix_time = time()
        port = addr[1]
        event = {
            'eventid': 'siphoney.request',
            'timestamp': get_utc_time(unix_time),
            'unixtime': unix_time,
            'src_ip': ip,
            'src_port': port,
            'dst_port': self.cfg['port'],
            'sensor': self.cfg['sensor'],
            'dst_ip': self.cfg['public_ip'] if self.cfg['report_public_ip'] else get_local_ip(),
            'message': message,
            'headers': headers
        }
        msg('{} from {}:{}.'.format(message, ip, self.cfg['port']))
        write_event(event, self.cfg)
